@@ -20,7 +20,8 @@
 
 #include <config.h>
 #include <string.h>
-
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
@@ -29,18 +30,21 @@
 #include "gossip-utils.h"
 #include "gossip-account-manager.h"
 
+#define RESOURCE_HACK /* For splitting an id of user@server/resource
+		       * to just user@server with resource in it's own
+		       * xml tags (for Gossip release 0.10.2).
+		       */
+
 #define ACCOUNTS_XML_FILENAME "accounts.xml"
 #define ACCOUNTS_DTD_FILENAME "gossip-account.dtd"
 
-#define DEBUG_MSG(x) 
+#define DEBUG_MSG(x)
 /* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_MANAGER, GossipAccountManagerPriv))
 
-
 typedef struct _GossipAccountManagerPriv GossipAccountManagerPriv;
 
-
 struct _GossipAccountManagerPriv {
 	GList *accounts;
 
@@ -50,7 +54,6 @@ struct _GossipAccountManagerPriv {
 	gchar *default_name_override;
 };
 
-
 enum {
 	ACCOUNT_ADDED,
 	ACCOUNT_REMOVED, 
@@ -58,20 +61,20 @@ enum {
 	LAST_SIGNAL
 };
 
-
-static void     account_manager_finalize             (GObject               *object);
-static gboolean account_manager_get_all              (GossipAccountManager  *manager);
-static gboolean account_manager_file_parse           (GossipAccountManager  *manager,
-						      const gchar           *filename);
-static gboolean account_manager_file_save            (GossipAccountManager  *manager);
-
+static void     account_manager_finalize   (GObject              *object);
+static gboolean account_manager_get_all    (GossipAccountManager *manager);
+static gboolean account_manager_file_parse (GossipAccountManager *manager,
+					    const gchar          *filename);
+static gboolean account_manager_file_save  (GossipAccountManager *manager);
 
 static guint signals[LAST_SIGNAL] = {0};
 
+#ifdef RESOURCE_HACK
+gboolean need_saving = FALSE;
+#endif
 
 G_DEFINE_TYPE (GossipAccountManager, gossip_account_manager, G_TYPE_OBJECT);
 
-
 static void
 gossip_account_manager_class_init (GossipAccountManagerClass *klass)
 {
@@ -164,7 +167,7 @@ gossip_account_manager_add (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 
-	/* don't add more than once */
+	/* Don't add more than once */
  	if (!gossip_account_manager_find (manager, gossip_account_get_name (account))) { 
 		const gchar       *name;
 		GossipAccountType  type;
@@ -173,7 +176,7 @@ gossip_account_manager_add (GossipAccountManager *manager,
 		name = gossip_account_get_name (account);
 
 		DEBUG_MSG (("Account Manager: Adding %s account with name:'%s'", 
-			   gossip_account_get_type_as_str (type), 
+			   gossip_account_type_to_string (type), 
 			   name));
 		
 		priv->accounts = g_list_append (priv->accounts, g_object_ref (account));
@@ -263,6 +266,33 @@ gossip_account_manager_find (GossipAccountManager *manager,
 	return NULL;
 }	
 
+GossipAccount *
+gossip_account_manager_find_by_id (GossipAccountManager *manager,
+				   const gchar          *id)
+{
+	GossipAccountManagerPriv *priv;
+	GList                    *l;
+	
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+	
+	priv = GET_PRIV (manager);
+
+	for (l = priv->accounts; l; l = l->next) {
+		GossipAccount *account;
+		const gchar   *account_id;
+
+		account = l->data;
+		account_id = gossip_account_get_id (account);
+
+		if (strcmp (account_id, id) == 0) {
+			return account;
+		}
+	}
+
+	return NULL;
+}	
+
 void
 gossip_account_manager_set_overridden_default (GossipAccountManager *manager,
 					       const gchar          *name)
@@ -410,7 +440,7 @@ account_manager_parse_account (GossipAccountManager *manager,
 	gchar             *str;
 	GossipAccountType  type;
 	gchar             *name, *id, *password;
-	gchar             *server;
+	gchar             *server, *resource;
 	guint16            port;
 	gboolean           auto_connect, use_ssl, use_proxy;
 
@@ -419,6 +449,7 @@ account_manager_parse_account (GossipAccountManager *manager,
 	name = NULL;
 	id = NULL;
 	password = NULL;
+	resource = NULL;
 	server = NULL;
 	port = 5222;
 	auto_connect = TRUE;
@@ -435,20 +466,22 @@ account_manager_parse_account (GossipAccountManager *manager,
 		if (strcmp (tag, "type") == 0) {
 			if (strcmp (str, "jabber") == 0) {
 				type = GOSSIP_ACCOUNT_TYPE_JABBER;
-				xmlFree (str);
 			}
 		}
 		else if (strcmp (tag, "name") == 0) {
-			name = str;
+			name = g_strdup (str);
 		}
 		else if (strcmp (tag, "id") == 0) {
-			id = str;
+			id = g_strdup (str);
 		}
 		else if (strcmp (tag, "password") == 0) {
-			password = str;
+			password = g_strdup (str);
 		}
 		else if (strcmp (tag, "server") == 0) {
-			server = str;
+			server = g_strdup (str);
+		}
+		else if (strcmp (tag, "resource") == 0) {
+			resource = g_strdup (str);
 		}
 		else if (strcmp (tag, "port") == 0) {
 			guint tmp_port;
@@ -457,7 +490,6 @@ account_manager_parse_account (GossipAccountManager *manager,
 			if (tmp_port != 0) {
 				port = tmp_port;
 			}
-			xmlFree (str);
 		}
 		else if (strcmp (tag, "auto_connect") == 0) {
 			if (strcmp (str, "yes") == 0) {
@@ -465,7 +497,6 @@ account_manager_parse_account (GossipAccountManager *manager,
 			} else {
 				auto_connect = FALSE;
 			}
-			xmlFree (str);
 		}
 		else if (strcmp (tag, "use_ssl") == 0) {
 			if (strcmp (str, "yes") == 0) {
@@ -473,7 +504,6 @@ account_manager_parse_account (GossipAccountManager *manager,
 			} else {
 				use_ssl = FALSE;
 			}
-			xmlFree (str);
 		}
 		else if (strcmp (tag, "use_proxy") == 0) {
 			if (strcmp (str, "yes") == 0) {
@@ -481,9 +511,10 @@ account_manager_parse_account (GossipAccountManager *manager,
 			} else {
 				use_proxy = FALSE;
 			}
-			xmlFree (str);
 		}
 
+		xmlFree (str);
+
 		child = child->next;
 	}
 
@@ -498,6 +529,10 @@ account_manager_parse_account (GossipAccountManager *manager,
 					"use_proxy", use_proxy,
 					NULL);
 		
+		if (resource) {
+			gossip_account_set_resource (account, resource);
+		}
+
 		if (server) {
 			gossip_account_set_server (account, server);
 		}
@@ -506,14 +541,43 @@ account_manager_parse_account (GossipAccountManager *manager,
 			gossip_account_set_password (account, password);
 		}
 		
+#ifdef RESOURCE_HACK
+		const gchar *resource_found = NULL;
+
+		if (id) {
+			/* FIXME: This hack is so we don't get bug
+			 * reports and basically gets the resource
+			 * from the id.
+			 */
+			const gchar *ch;
+			
+			ch = strchr (id, '/');
+			if (ch) {
+				resource_found = (ch + 1);
+				id[ch - id] = '\0';
+
+				g_printerr ("Converting ID... (id:'%s', resource:'%s')\n", 
+					    id, resource_found);
+			}
+		}
+
+		if (resource_found) {
+			gossip_account_set_id (account, id);
+			gossip_account_set_resource (account, resource_found);
+			need_saving = TRUE;
+		}
+#endif /* RESOURCE_HACK */
+
 		gossip_account_manager_add (manager, account);
 		
 		g_object_unref (account);
 	}
 	
-	xmlFree (name);
-	xmlFree (id);
-	xmlFree (server);
+	g_free (name);
+	g_free (id);
+	g_free (password);
+	g_free (resource);
+	g_free (server);
 }
 
 static gboolean
@@ -559,6 +623,7 @@ account_manager_file_parse (GossipAccountManager *manager,
 
 			g_free (priv->default_name);
 			priv->default_name = g_strdup (str);
+
 			xmlFree (str);
 		}
 		else if (strcmp ((gchar *) node->name, "account") == 0) {
@@ -577,6 +642,13 @@ account_manager_file_parse (GossipAccountManager *manager,
 	xmlFreeDoc(doc);
 	xmlFreeParserCtxt (ctxt);
 	
+#ifdef RESOURCE_HACK
+	if (need_saving) {
+		g_printerr ("Saving accounts... \n");
+		account_manager_file_save (manager);
+	}
+#endif
+
 	return TRUE;
 }
 
@@ -592,7 +664,8 @@ account_manager_file_save (GossipAccountManager *manager)
 	gchar                    *dtd_file;
 	gchar                    *xml_dir;
 	gchar                    *xml_file;
-					
+	mode_t                    old_mask;
+
 	priv = GET_PRIV (manager);
 	
 	if (priv->accounts_file_name) {
@@ -646,6 +719,7 @@ account_manager_file_save (GossipAccountManager *manager)
 		xmlNewTextChild (node, NULL, "name", gossip_account_get_name (account));
 		xmlNewTextChild (node, NULL, "id", gossip_account_get_id (account));
 
+		xmlNewTextChild (node, NULL, "resource", gossip_account_get_resource (account));
 		xmlNewTextChild (node, NULL, "password", gossip_account_get_password (account));
 		xmlNewTextChild (node, NULL, "server", gossip_account_get_server (account));
 		xmlNewChild (node, NULL, "port", port);
@@ -659,13 +733,20 @@ account_manager_file_save (GossipAccountManager *manager)
 	}
 
 	DEBUG_MSG (("Account Manager: Saving file:'%s'", xml_file));
+
+	/* Set the umask to get the proper permissions when libxml saves the
+	 * file, but also change the permissions expiicitly in case the file
+	 * already exists.
+	 */
+	old_mask = umask (077);
+	chmod (xml_file, 0600);
 	xmlSaveFormatFileEnc (xml_file, doc, "utf-8", 1);
+	umask (old_mask);
+
 	xmlFreeDoc (doc);
-
 	xmlCleanupParser ();
-
 	xmlMemoryDump ();
-	
+
 	g_free (xml_file);
 
 	return TRUE;
