@@ -236,7 +236,7 @@ gossip_chatroom_manager_add (GossipChatroomManager *manager,
 
 		DEBUG_MSG (("ChatroomManager: Adding %s%s chatroom with name:'%s'", 
 			   gossip_chatroom_get_auto_connect (chatroom) ? "connecting on startup " : "", 
-			   gossip_chatroom_get_type_as_str (type), 
+			   gossip_chatroom_type_to_string (type), 
 			   name));
 		
 		g_signal_connect (chatroom, "notify::enabled", 
@@ -368,7 +368,6 @@ gossip_chatroom_manager_find (GossipChatroomManager *manager,
 
 GList *
 gossip_chatroom_manager_find_extended (GossipChatroomManager *manager,
-				       const gchar           *nick,
 				       const gchar           *server,
 				       const gchar           *room)
 {
@@ -377,7 +376,6 @@ gossip_chatroom_manager_find_extended (GossipChatroomManager *manager,
 	GList                     *l;
 
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager), NULL);
-	g_return_val_if_fail (nick != NULL, NULL);
 	g_return_val_if_fail (server != NULL, NULL);
 	g_return_val_if_fail (room != NULL, NULL);
 	
@@ -385,18 +383,15 @@ gossip_chatroom_manager_find_extended (GossipChatroomManager *manager,
 
 	for (l = priv->chatrooms; l; l = l->next) {
 		GossipChatroom *chatroom;
-		const gchar    *chatroom_nick;
 		const gchar    *chatroom_server;
 		const gchar    *chatroom_room;
 
 		chatroom = l->data;
 		
-		chatroom_nick = gossip_chatroom_get_nick (chatroom);
 		chatroom_server = gossip_chatroom_get_server (chatroom);
 		chatroom_room = gossip_chatroom_get_room (chatroom);
 		
-		if (strcmp (nick, chatroom_nick) == 0 &&
-		    strcmp (server, chatroom_server) == 0 &&
+		if (strcmp (server, chatroom_server) == 0 &&
 		    strcmp (room, chatroom_room) == 0) {
 			found = g_list_append (found, g_object_ref (chatroom));
 		}
@@ -546,22 +541,21 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 
 		if (strcmp (tag, "type") == 0) {
 			/* we have no types yet */
-			xmlFree (str);
 		}
 		else if (strcmp (tag, "name") == 0) {
-			name = str;
+			name = g_strdup (str);
 		}
 		else if (strcmp (tag, "nick") == 0) {
-			nick = str;
+			nick = g_strdup (str);
 		}
 		else if (strcmp (tag, "server") == 0) {
-			server = str;
+			server = g_strdup (str);
 		}
 		else if (strcmp (tag, "room") == 0) {
-			room = str;
+			room = g_strdup (str);
 		}
 		else if (strcmp (tag, "password") == 0) {
-			password = str;
+			password = g_strdup (str);
 		}
 		else if (strcmp (tag, "auto_connect") == 0) {
 			if (strcmp (str, "yes") == 0) {
@@ -569,12 +563,13 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 			} else {
 				auto_connect = FALSE;
 			}
-			xmlFree (str);
 		}
 		else if (strcmp (tag, "account") == 0) {
-			account_name = str;
+			account_name = g_strdup (str);
 		}
 
+		xmlFree (str);
+
 		child = child->next;
 	}
 
@@ -613,12 +608,12 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 		g_object_unref (chatroom);
 	}
 	
-	xmlFree (name);
-	xmlFree (nick);
-	xmlFree (server);
-	xmlFree (room);
-	xmlFree (password);
-	xmlFree (account_name);
+	g_free (name);
+	g_free (nick);
+	g_free (server);
+	g_free (room);
+	g_free (password);
+	g_free (account_name);
 }
 
 static gboolean
