@@ -23,6 +23,7 @@
 
 #include <glib-object.h>
 
+#include "gossip-account.h"
 #include "gossip-presence.h"
 
 #define GOSSIP_TYPE_CONTACT         (gossip_contact_get_gtype ())
@@ -32,11 +33,9 @@
 #define GOSSIP_IS_CONTACT_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_CONTACT))
 #define GOSSIP_CONTACT_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_CONTACT, GossipContactClass))
 
-
 typedef struct _GossipContact      GossipContact;
 typedef struct _GossipContactClass GossipContactClass;
 
-
 struct _GossipContact {
 	GObject parent;
 };
@@ -46,7 +45,6 @@ struct _GossipContactClass {
 	GObjectClass parent_class;
 };
 
-
 typedef enum {
 	GOSSIP_CONTACT_TYPE_TEMPORARY,
 	GOSSIP_CONTACT_TYPE_CONTACTLIST,
@@ -54,7 +52,6 @@ typedef enum {
 	GOSSIP_CONTACT_TYPE_USER /* Represents the own user */
 } GossipContactType;
 
-
 typedef enum {
 	GOSSIP_SUBSCRIPTION_NONE,
 	GOSSIP_SUBSCRIPTION_TO,
@@ -62,20 +59,19 @@ typedef enum {
 	GOSSIP_SUBSCRIPTION_BOTH
 } GossipSubscription;
 
-
 GType              gossip_contact_get_gtype                 (void) G_GNUC_CONST;
 
-GossipContact *    gossip_contact_new                       (GossipContactType   type);
+GossipContact *    gossip_contact_new                       (GossipContactType   type,
+							     GossipAccount      *account);
 GossipContact *    gossip_contact_new_full                  (GossipContactType   type,
+							     GossipAccount      *account,
 							     const gchar        *id,
 							     const gchar        *name);
+GossipContact *    gossip_contact_copy                      (GossipContact      *contact);
 GossipContactType  gossip_contact_get_type                  (GossipContact      *contact);
 const gchar *      gossip_contact_get_id                    (GossipContact      *contact);
-void               gossip_contact_set_id                    (GossipContact      *contact,
-							     const gchar        *id);
-void               gossip_contact_set_name                  (GossipContact      *contact,
-							     const gchar        *name);
 const gchar *      gossip_contact_get_name                  (GossipContact      *contact);
+GossipAccount *    gossip_contact_get_account               (GossipContact      *contact);
 void               gossip_contact_add_presence              (GossipContact      *contact,
 							     GossipPresence     *presence);
 void               gossip_contact_remove_presence           (GossipContact      *contact,
@@ -84,11 +80,16 @@ GossipPresence *   gossip_contact_get_presence_for_resource (GossipContact
 							     const gchar        *resource);
 GossipPresence *   gossip_contact_get_active_presence       (GossipContact      *contact);
 GList *            gossip_contact_get_presence_list         (GossipContact      *contact);
-gboolean           gossip_contact_is_online                 (GossipContact      *contact);
-gboolean           gossip_contact_set_groups                (GossipContact      *contact,
-							     GList              *categories);
 GList *            gossip_contact_get_groups                (GossipContact      *contact);
 GossipSubscription gossip_contact_get_subscription          (GossipContact      *contact);
+void               gossip_contact_set_id                    (GossipContact      *contact,
+							     const gchar        *id);
+void               gossip_contact_set_name                  (GossipContact      *contact,
+							     const gchar        *name);
+void               gossip_contact_set_account               (GossipContact      *contact,
+							     GossipAccount      *account);
+gboolean           gossip_contact_set_groups                (GossipContact      *contact,
+							     GList              *categories);
 void               gossip_contact_set_subscription          (GossipContact      *contact,
 							     GossipSubscription  subscription);
 gint               gossip_contact_compare                   (gconstpointer       a,
@@ -103,8 +104,8 @@ gint               gossip_contact_name_case_n_compare       (gconstpointer
 gboolean           gossip_contact_equal                     (gconstpointer       v1,
 							     gconstpointer       v2);
 guint              gossip_contact_hash                      (gconstpointer       key);
+gboolean           gossip_contact_is_online                 (GossipContact      *contact);
 const gchar *      gossip_contact_get_status                (GossipContact      *contact);
 
-
 #endif /* __GOSSIP_CONTACT_H__ */
 
