@@ -24,14 +24,13 @@
 
 #include "gossip-session.h"
 
-#define d(x)
+#define DEBUG_MSG(x) 
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_SESSION, GossipSessionPriv))
 
-
 typedef struct _GossipSessionPriv  GossipSessionPriv;
 
-
 struct _GossipSessionPriv {
  	GossipAccountManager *account_manager; 
 
@@ -47,32 +46,26 @@ struct _GossipSessionPriv {
 	GHashTable           *timers; /* connected time */
 };
 
-
-typedef struct {
-	gchar         *contact_id;
+struct FindAccount {
+	const gchar   *contact_id;
 	GossipAccount *account;
-} FindAccount;
-
+};
 
 struct CountAccounts {
 	GossipSession *session;
 	guint          connected;
-	guint          connected_total;
 	guint          disconnected;
 };
 
-
-typedef struct {
+struct GetAccounts {
 	GossipSession *session;
 	GList         *accounts;
-} GetAccounts;
+};
 
-
-typedef struct {
+struct ConnectData {
 	GossipSession *session;
 	gboolean       startup;
-} ConnectAccounts;
-
+};
 
 static void            gossip_session_class_init                 (GossipSessionClass   *klass);
 static void            gossip_session_init                       (GossipSession        *session);
@@ -100,7 +93,7 @@ static void            session_protocol_contact_presence_updated (GossipProtocol
 static void            session_protocol_contact_removed          (GossipProtocol       *protocol,
 								  GossipContact        *contact,
 								  GossipSession        *session);
-static void            session_protocol_composing_event          (GossipProtocol       *protocol,
+static void            session_protocol_composing                (GossipProtocol       *protocol,
 								  GossipContact        *contact,
 								  gboolean              composing,
 								  GossipSession        *session);
@@ -115,13 +108,13 @@ static GossipProtocol *session_get_protocol                      (GossipSession
 								  GossipContact        *contact);
 static void            session_get_accounts_foreach_cb           (GossipAccount        *account,
 								  GossipProtocol       *protocol,
-								  GetAccounts          *data);
+								  struct GetAccounts   *ga);
 static void            session_count_accounts_foreach_cb         (GossipAccount        *account,
 								  GossipProtocol       *protocol,
 								  struct CountAccounts *ca);
 static void            session_find_account_foreach_cb           (GossipAccount        *account,
 								  GossipProtocol       *protocol,
-								  FindAccount          *fa);
+								  struct FindAccount   *fa);
 static void            session_account_added_cb                  (GossipAccountManager *manager,
 								  GossipAccount        *account,
 								  GossipSession        *session);
@@ -132,15 +125,13 @@ static void            session_connect                           (GossipSession
 								  GossipAccount        *account);
 static void            session_connect_foreach_cb                (GossipAccount        *account,
 								  GossipProtocol       *protocol,
-								  ConnectAccounts      *data);
+								  struct ConnectData   *cd);
 static void            session_disconnect                        (GossipSession        *session,
 								  GossipAccount        *account);
 static void            session_disconnect_foreach_cb             (GossipAccount        *account,
 								  GossipProtocol       *protocol,
 								  GossipSession        *session);
 	
-
-/* signals */
 enum {
 	CONNECTING,
 	CONNECTED,
@@ -157,20 +148,17 @@ enum {
 	CONTACT_UPDATED,
 	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
-	COMPOSING_EVENT,
+	COMPOSING,
 
 	/* Used for protocols to retreive information from UI */
 	GET_PASSWORD,
 	LAST_SIGNAL
 };
 
-
-G_DEFINE_TYPE (GossipSession, gossip_session, G_TYPE_OBJECT);
-
-
 static guint    signals[LAST_SIGNAL] = {0};
 static gpointer parent_class;
 
+G_DEFINE_TYPE (GossipSession, gossip_session, G_TYPE_OBJECT);
 
 static void
 gossip_session_class_init (GossipSessionClass *klass)
@@ -225,9 +213,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
 	signals[PROTOCOL_CONNECTED] = 
 		g_signal_new ("protocol-connected",
@@ -235,9 +223,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_POINTER,
+			      libgossip_marshal_VOID__OBJECT_OBJECT,
 			      G_TYPE_NONE, 
-			      2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      2, GOSSIP_TYPE_ACCOUNT, GOSSIP_TYPE_PROTOCOL);
 	
 	signals[PROTOCOL_DISCONNECTED] = 
 		g_signal_new ("protocol-disconnected",
@@ -245,9 +233,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_POINTER,
+			      libgossip_marshal_VOID__OBJECT_OBJECT,
 			      G_TYPE_NONE, 
-			      2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      2, GOSSIP_TYPE_ACCOUNT, GOSSIP_TYPE_PROTOCOL);
 
 	signals[PROTOCOL_DISCONNECTING] = 
 		g_signal_new ("protocol-disconnecting",
@@ -255,9 +243,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 	
 	signals[PROTOCOL_ERROR] = 
 		g_signal_new ("protocol-error",
@@ -265,9 +253,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_POINTER_POINTER,
+			      libgossip_marshal_VOID__OBJECT_OBJECT_POINTER,
 			      G_TYPE_NONE, 
-			      3, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_POINTER);
+			      3, GOSSIP_TYPE_PROTOCOL, GOSSIP_TYPE_ACCOUNT, G_TYPE_POINTER);
 
 	signals[NEW_MESSAGE] = 
 		g_signal_new ("new-message",
@@ -275,9 +263,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_MESSAGE);
 
 	signals[PRESENCE_CHANGED] = 
 		g_signal_new ("presence-changed",
@@ -285,9 +273,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_PRESENCE);
 
 	signals[CONTACT_ADDED] = 
 		g_signal_new ("contact-added",
@@ -295,18 +283,18 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 	signals[CONTACT_UPDATED] = 
 		g_signal_new ("contact-updated",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_PRESENCE_UPDATED] = 
 		g_signal_new ("contact-presence-updated",
@@ -314,9 +302,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_REMOVED] = 
 		g_signal_new ("contact-removed",
@@ -324,27 +312,27 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
-	signals[COMPOSING_EVENT] =
-		g_signal_new ("composing-event",
+			      1, GOSSIP_TYPE_CONTACT);
+	signals[COMPOSING] =
+		g_signal_new ("composing",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_BOOLEAN,
+			      libgossip_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+			      2, GOSSIP_TYPE_CONTACT, G_TYPE_BOOLEAN);
 	signals[GET_PASSWORD] =
 		g_signal_new ("get-password",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_STRING__POINTER,
+			      libgossip_marshal_STRING__OBJECT,
 			      G_TYPE_STRING,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 	
 	g_type_class_add_private (object_class, sizeof (GossipSessionPriv));
 }
@@ -443,8 +431,8 @@ session_protocol_signals_setup (GossipSession  *session,
 	g_signal_connect (protocol, "contact-removed",
 			  G_CALLBACK (session_protocol_contact_removed),
 			  session);
-	g_signal_connect (protocol, "composing-event",
-			  G_CALLBACK (session_protocol_composing_event),
+	g_signal_connect (protocol, "composing",
+			  G_CALLBACK (session_protocol_composing),
 			  session);
 	g_signal_connect (protocol, "get-password",
 			  G_CALLBACK (session_protocol_get_password),
@@ -462,7 +450,7 @@ session_protocol_logged_in (GossipProtocol *protocol,
 	GossipSessionPriv *priv;
 	GTimer            *timer;
 
-	d(g_print ("Session: Protocol logged in\n"));
+	DEBUG_MSG (("Session: Protocol logged in"));
 
 	priv = GET_PRIV (session);
 
@@ -494,20 +482,23 @@ session_protocol_logged_out (GossipProtocol *protocol,
 	gdouble            seconds;
 
 	seconds = gossip_session_get_connected_time (session, account);
-	d(g_print ("Session: Protocol logged out (after %.2f seconds)\n", seconds));
+	DEBUG_MSG (("Session: Protocol logged out (after %.2f seconds)", seconds));
 
 	priv = GET_PRIV (session);
 
-	/* remove timer */
+	/* Remove timer */
 	g_hash_table_remove (priv->timers, account);
 
-	/* update some status? */
+	/* Update some status? */
 	if (priv->connected_counter < 0) {
 		g_warning ("We have some issues in the connection counting");
 		return;
-	}
+	} 
 
-	priv->connected_counter--;
+	/* Don't go lower than 0 */
+	if (priv->connected_counter > 0) {
+		priv->connected_counter--;
+	}
 	
 	g_signal_emit (session, signals[PROTOCOL_DISCONNECTED], 0, account, protocol);
 	
@@ -533,7 +524,7 @@ session_protocol_contact_added (GossipProtocol *protocol,
 {
 	GossipSessionPriv *priv;
 
-	d(g_print ("Session: Contact added '%s'\n",
+	DEBUG_MSG (("Session: Contact added '%s'",
 		   gossip_contact_get_name (contact)));
 
 	priv = GET_PRIV (session);
@@ -549,7 +540,7 @@ session_protocol_contact_updated (GossipProtocol *protocol,
 				  GossipContact  *contact,
 				  GossipSession  *session)
 {
-	d(g_print ("Session: Contact updated '%s'\n",
+	DEBUG_MSG (("Session: Contact updated '%s'",
 		   gossip_contact_get_name (contact)));
 
 	g_signal_emit (session, signals[CONTACT_UPDATED], 0, contact);
@@ -560,7 +551,7 @@ session_protocol_contact_presence_updated (GossipProtocol *protocol,
 					   GossipContact  *contact,
 					   GossipSession  *session)
 {
-	d(g_print ("Session: Contact presence updated '%s'\n",
+	DEBUG_MSG (("Session: Contact presence updated '%s'",
 		   gossip_contact_get_name (contact)));
 	g_signal_emit (session, signals[CONTACT_PRESENCE_UPDATED], 0, contact);
 }
@@ -572,7 +563,7 @@ session_protocol_contact_removed (GossipProtocol *protocol,
 {
 	GossipSessionPriv *priv;
 	
-	d(g_print ("Session: Contact removed '%s'\n",
+	DEBUG_MSG (("Session: Contact removed '%s'",
 		   gossip_contact_get_name (contact)));
 
 	priv = GET_PRIV (session);
@@ -584,20 +575,20 @@ session_protocol_contact_removed (GossipProtocol *protocol,
 }
 
 static void 
-session_protocol_composing_event (GossipProtocol *protocol,
-				  GossipContact  *contact,
-				  gboolean        composing,
-				  GossipSession  *session)
+session_protocol_composing (GossipProtocol *protocol,
+			    GossipContact  *contact,
+			    gboolean        composing,
+			    GossipSession  *session)
 {
 	GossipSessionPriv *priv;
 	
-	d(g_print ("Session: Contact %s composing:'%s'\n",
+	DEBUG_MSG (("Session: Contact %s composing:'%s'",
 		   composing ? "is" : "is not",
 		   gossip_contact_get_name (contact)));
 
 	priv = GET_PRIV (session);
 	
-	g_signal_emit (session, signals[COMPOSING_EVENT], 0, contact, composing);
+	g_signal_emit (session, signals[COMPOSING], 0, contact, composing);
 }
 
 static gchar *
@@ -605,9 +596,9 @@ session_protocol_get_password (GossipProtocol *protocol,
 			       GossipAccount  *account,
 			       GossipSession  *session)
 {
-	gchar *password;
+	gchar *password = NULL;
 
-	d(g_print ("Session: Get password\n"));
+	DEBUG_MSG (("Session: Get password"));
 
 	g_signal_emit (session, signals[GET_PASSWORD], 0, account, &password);
 	
@@ -621,7 +612,7 @@ session_protocol_error (GossipProtocol *protocol,
 			GossipSession  *session)
 {
 
-	d(g_print ("Session: Error:%d->'%s'\n", 
+	DEBUG_MSG (("Session: Error:%d->'%s'", 
 		   error->code, error->message));
 
 	g_signal_emit (session, signals[PROTOCOL_ERROR], 0, protocol, account, error); 
@@ -726,39 +717,36 @@ gossip_session_get_account_manager (GossipSession *session)
 GList *
 gossip_session_get_accounts (GossipSession *session)
 {
-	GossipSessionPriv *priv;
-	GetAccounts       *data;
-	GList             *list;
+	GossipSessionPriv  *priv;
+	struct GetAccounts  ga;
+	GList              *list;
 
 	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
 	
 	priv = GET_PRIV (session);
 
-	data = g_new0 (GetAccounts, 1);
-	data->session = g_object_ref (session);
+	ga.session = session;
+	ga.accounts = NULL;
 
 	g_hash_table_foreach (priv->accounts, 
 			      (GHFunc)session_get_accounts_foreach_cb,
-			      data);
+			      &ga);
 
-	list = data->accounts;
-
-	g_object_unref (data->session);
-	g_free (data);
+	list = ga.accounts;
 
 	return list;
 }
 
 static void
-session_get_accounts_foreach_cb (GossipAccount   *account,
-				 GossipProtocol  *protocol,
-				 GetAccounts     *data)
+session_get_accounts_foreach_cb (GossipAccount      *account,
+				 GossipProtocol     *protocol,
+				 struct GetAccounts *ga)
 {
 	GossipSessionPriv *priv;
 
-	priv = GET_PRIV (data->session);
+	priv = GET_PRIV (ga->session);
 
-	data->accounts = g_list_append (data->accounts, g_object_ref (account));
+	ga->accounts = g_list_append (ga->accounts, g_object_ref (account));
 }
 
 gdouble
@@ -785,7 +773,6 @@ gossip_session_get_connected_time (GossipSession *session,
 void 
 gossip_session_count_accounts (GossipSession *session,
 			       guint         *connected,
-			       guint         *connected_total,
 			       guint         *disconnected)
 {
 	GossipSessionPriv    *priv;
@@ -798,7 +785,6 @@ gossip_session_count_accounts (GossipSession *session,
 	ca.session = session;
 
 	ca.connected = 0;
-	ca.connected_total = 0;
 	ca.disconnected = 0;
 
 	g_hash_table_foreach (priv->accounts, 
@@ -809,10 +795,6 @@ gossip_session_count_accounts (GossipSession *session,
 		*connected = ca.connected;
 	}
 
-	if (connected_total) {
-		*connected_total = ca.connected_total;
-	}
-
 	if (disconnected) {
 		*disconnected = ca.disconnected;
 	}
@@ -828,15 +810,9 @@ session_count_accounts_foreach_cb (GossipAccount        *account,
 	priv = GET_PRIV (ca->session);
 
 	if (gossip_protocol_is_connected (protocol)) {
-		if (gossip_account_get_enabled (account)) {
-			ca->connected++;
-		}
-
-		ca->connected_total++;
+		ca->connected++;
 	} else {
-		if (gossip_account_get_enabled (account)) {
-			ca->disconnected++;
-		}
+		ca->disconnected++;
 	}
 }
 
@@ -915,37 +891,28 @@ GossipAccount *
 gossip_session_find_account (GossipSession *session,
 			     GossipContact *contact)
 {
-	GossipSessionPriv *priv;
-	GossipAccount     *account = NULL;
-	FindAccount       *fa;
+	GossipSessionPriv  *priv;
+	struct FindAccount  fa;
 
 	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
 	
 	priv = GET_PRIV (session);
 
-	fa = g_new0 (FindAccount, 1);
-	
-	fa->contact_id = g_strdup (gossip_contact_get_id (contact));
+	fa.contact_id = gossip_contact_get_id (contact);
+	fa.account = NULL;
 
 	g_hash_table_foreach (priv->accounts, 
 			      (GHFunc)session_find_account_foreach_cb,
-			      fa);
+			      &fa);
 
-	if (fa->account) {
-		account = fa->account;
-	}
-
-	g_free (fa->contact_id);
-	g_free (fa);
-
-	return account;
+	return fa.account;
 }
 
 static void
-session_find_account_foreach_cb (GossipAccount  *account,
-				 GossipProtocol *protocol,
-				 FindAccount    *fa)
+session_find_account_foreach_cb (GossipAccount      *account,
+				 GossipProtocol     *protocol,
+				 struct FindAccount *fa)
 {
 	if (gossip_protocol_find_contact (protocol, fa->contact_id)) {
 		fa->account = g_object_ref (account);
@@ -973,8 +940,8 @@ gossip_session_connect (GossipSession *session,
 			GossipAccount *account,
 			gboolean       startup)
 {
-	GossipSessionPriv *priv;
-	ConnectAccounts   *data;
+	GossipSessionPriv  *priv;
+	struct ConnectData  cd;
 	
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
 
@@ -984,13 +951,13 @@ gossip_session_connect (GossipSession *session,
 
 	g_signal_emit (session, signals[CONNECTING], 0);
 
-	/* temporary */
+	/* Temporary */
 	if (!priv->presence) {
 		priv->presence = gossip_presence_new_full (GOSSIP_PRESENCE_STATE_AVAILABLE, 
 							   NULL);
 	}
 
-	/* connect one account if provided */
+	/* Connect one account if provided */
 	if (account) {
 		g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 		
@@ -998,18 +965,13 @@ gossip_session_connect (GossipSession *session,
 		return;
 	}
 
-	/* connect ALL accounts if no one account is specified */
-	data = g_new0 (ConnectAccounts, 1);
-
-	data->session = g_object_ref (session);
-	data->startup = startup;
+	cd.session = session;
+	cd.startup = startup;
 
+	/* Connect ALL accounts if no one account is specified */
 	g_hash_table_foreach (priv->accounts,
-			      (GHFunc)session_connect_foreach_cb,
-			      data);
-
-	g_object_unref (data->session);
-	g_free (data);
+			      (GHFunc) session_connect_foreach_cb,
+			      &cd);
 }
 
 static void
@@ -1037,24 +999,16 @@ session_connect (GossipSession *session,
 }
 
 static void
-session_connect_foreach_cb (GossipAccount   *account,
-			    GossipProtocol  *protocol,
-			    ConnectAccounts *data)
+session_connect_foreach_cb (GossipAccount      *account,
+			    GossipProtocol     *protocol,
+			    struct ConnectData *cd)
 {
-	GossipSessionPriv *priv;
+ 	if (cd->startup && 
+	    !gossip_account_get_auto_connect (account)) { 
+ 		return; 
+ 	} 
 
-	priv = GET_PRIV (data->session);
-
-	if (!gossip_account_get_enabled (account)) {
-		return;
-	}
-	
-	if (data->startup && 
-	    !gossip_account_get_auto_connect (account)) {
-		return;
-	}
-
-	session_connect (data->session, account);
+	session_connect (cd->session, account);
 }
 
 void
@@ -1095,10 +1049,6 @@ session_disconnect (GossipSession *session,
 	
 	protocol = g_hash_table_lookup (priv->accounts, account);
 	
-/* 	if (!gossip_protocol_is_connected (protocol)) { */
-/* 		return; */
-/* 	} */
-
 	g_signal_emit (session, signals[PROTOCOL_DISCONNECTING], 0, account);
 	
 	gossip_protocol_logout (protocol);
@@ -1112,14 +1062,6 @@ session_disconnect_foreach_cb (GossipAccount  *account,
 	GossipSessionPriv *priv;
 
 	priv = GET_PRIV (session);
-
-	/* don't disconnect accounts which are disabled since
-	   they might have been started manually for other
-	   reasons and we only want to disconnect all accounts
-	   we would normall connect */
-	if (!gossip_account_get_enabled (account)) {
-		return;
-	}
 	
 	session_disconnect (session, account);
 }
@@ -1179,7 +1121,14 @@ gossip_session_send_composing (GossipSession  *session,
 	priv = GET_PRIV (session);
 
 	protocol = session_get_protocol (session, contact);
-	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+	if (!protocol) {
+		/* We don't warn here because if no protocol is found,
+		 * it is likely that it is because the contact is a
+		 * temporary contact, instead we just silently ignore
+		 * it and don't send composing. 
+		 */
+		return;
+	}
 
 	if (!gossip_protocol_is_connected (protocol)) {
 		return;
@@ -1246,7 +1195,7 @@ gossip_session_is_connected (GossipSession *session,
 		return gossip_protocol_is_connected (protocol);
 	} 
 
-	/* fall back to counter if no account is provided */
+	/* Fall back to counter if no account is provided */
 	return (priv->connected_counter > 0);
 }
 
@@ -1259,8 +1208,9 @@ gossip_session_get_active_resource (GossipSession *session,
 
 	g_return_val_if_fail (GOSSIP_IS_SESSION (session), NULL);
 	
-	/* get the activate resource, needed to be able to lock the
-	   chat against a certain resource */
+	/* Get the activate resource, needed to be able to lock the
+	 * chat against a certain resource.
+	 */
 
 	priv = GET_PRIV (session);
 
@@ -1398,8 +1348,9 @@ gossip_session_remove_contact (GossipSession *session,
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
         g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 	
-	/* get the activate resource, needed to be able to lock the
-	   chat against a certain resource */
+	/* Get the activate resource, needed to be able to lock the
+	 * chat against a certain resource.
+	 */
 
 	priv = GET_PRIV (session);
 
@@ -1518,7 +1469,6 @@ gossip_session_get_own_contact (GossipSession *session,
 	return gossip_protocol_get_own_contact (protocol);
 }
 
-
 GList *
 gossip_session_get_groups (GossipSession *session)
 {
