@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004-2005 Imendio AB
+ * Copyright (C) 2004-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,10 +19,8 @@
  */
 
 #include <config.h>
-
 #include <stdlib.h>
 #include <string.h>
-
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-utils.h>
@@ -32,23 +30,22 @@
 #include "gossip-jabber-utils.h"
 #include "gossip-jabber-private.h"
 
-#define d(x)
+#define DEBUG_MSG(x) 
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
 
-#define JOIN_TIMEOUT       20000
-
-#define JOIN_MSG_ID_PREFIX "gc_join_"
+#define XMPP_MUC_USER_XMLNS "http://jabber.org/protocol/muc#user"
 
+#define JOIN_TIMEOUT        20000
 
 struct _GossipJabberChatrooms {
-	GossipJabber   *jabber;
-	GossipPresence *presence;
-	LmConnection   *connection;
+	GossipJabber          *jabber;
+	GossipPresence        *presence;
+	LmConnection          *connection;
 
-	GHashTable     *room_id_hash;
-	GHashTable     *room_jid_hash;
+	GHashTable            *room_id_hash;
+	GHashTable            *room_jid_hash;
 };
 
-
 typedef struct {
 	gint                   ref_count;
 
@@ -69,7 +66,9 @@ typedef struct {
 	LmMessageHandler      *join_handler;
 } JabberChatroom;
 
-
+static void             jabber_chatrooms_logged_out_cb        (GossipProtocol         *jabber,
+							       GossipAccount          *account,
+							       GossipJabberChatrooms  *chatrooms);
 static JabberChatroom * jabber_chatrooms_chatroom_new         (GossipJabberChatrooms  *chatrooms,
 							       GossipChatroom         *chatroom);
 static JabberChatroom * jabber_chatrooms_chatroom_ref         (JabberChatroom         *room);
@@ -97,7 +96,6 @@ static void             jabber_chatrooms_set_presence_foreach (gpointer
 							       JabberChatroom         *room,
 							       GossipJabberChatrooms  *chatrooms);
 
-
 GossipJabberChatrooms *
 gossip_jabber_chatrooms_init (GossipJabber *jabber)
 {
@@ -112,15 +110,20 @@ gossip_jabber_chatrooms_init (GossipJabber *jabber)
 
 	chatrooms = g_new0 (GossipJabberChatrooms, 1);
 	
-	chatrooms->jabber     = g_object_ref (jabber);
+	chatrooms->jabber = g_object_ref (jabber);
 	chatrooms->connection = lm_connection_ref (connection);
-	chatrooms->presence   = NULL;
+	chatrooms->presence = NULL;
+
 	chatrooms->room_id_hash = g_hash_table_new (NULL, NULL);
 	chatrooms->room_jid_hash = g_hash_table_new_full (gossip_jid_hash,
 							  gossip_jid_equal,
 							  (GDestroyNotify) gossip_jid_unref,
 							  NULL);
 
+	g_signal_connect (GOSSIP_PROTOCOL (chatrooms->jabber), "logged-out",
+			  G_CALLBACK (jabber_chatrooms_logged_out_cb), 
+			  chatrooms);
+
 	handler = lm_message_handler_new ((LmHandleMessageFunction) jabber_chatrooms_message_handler,
 					  chatrooms, NULL);
 
@@ -147,11 +150,16 @@ gossip_jabber_chatrooms_finalize (GossipJabberChatrooms *chatrooms)
 {
 	if (!chatrooms) {
 		/* We don't error here, because if no connection is
-		made, then we can clean up a GossipJabber object
-		without any chatrooms ever existing */
+		 * made, then we can clean up a GossipJabber object
+		 * without any chatrooms ever existing.
+		 */
 		return;
 	}
 
+	g_signal_handlers_disconnect_by_func (GOSSIP_PROTOCOL (chatrooms->jabber), 
+					      jabber_chatrooms_logged_out_cb, 
+					      chatrooms);
+
 	g_hash_table_destroy (chatrooms->room_id_hash);
 	g_hash_table_destroy (chatrooms->room_jid_hash);
 
@@ -165,6 +173,27 @@ gossip_jabber_chatrooms_finalize (GossipJabberChatrooms *chatrooms)
 	g_free (chatrooms);
 }
 
+static void
+jabber_chatrooms_logged_out_cb (GossipProtocol        *jabber,
+				GossipAccount         *account,
+				GossipJabberChatrooms *chatrooms)
+{
+	GList *rooms;
+	GList *l;
+
+	/* Clean up chat rooms */
+	rooms = gossip_jabber_chatrooms_get_rooms (chatrooms);
+
+	for (l = rooms; l; l = l->next) {
+		GossipChatroomId id;
+
+		id = GPOINTER_TO_INT (l->data);
+		gossip_jabber_chatrooms_leave (chatrooms, id);
+	}
+
+	g_list_free (rooms);
+}
+
 static JabberChatroom *
 jabber_chatrooms_chatroom_new (GossipJabberChatrooms *chatrooms,
 			       GossipChatroom        *chatroom)
@@ -181,9 +210,8 @@ jabber_chatrooms_chatroom_new (GossipJabberChatrooms *chatrooms,
 	room->ref_count = 1;
 	room->chatroom = g_object_ref (chatroom);
 
-	jid_str = g_strdup_printf ("%s@%s/%s", 
-				   gossip_chatroom_get_room (chatroom), 
-				   gossip_chatroom_get_server (chatroom),
+	jid_str = g_strdup_printf ("%s/%s", 
+				   gossip_chatroom_get_id_str (chatroom), 
 				   gossip_chatroom_get_nick (chatroom));
 	room->jid = gossip_jid_new (jid_str);
 	g_free (jid_str);
@@ -272,11 +300,13 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 {
 	GossipJID        *jid;
 	GossipMessage    *message;
+	GossipContact    *contact;
 	GossipChatroomId  id;
 	JabberChatroom   *room;
 	LmMessageNode    *node;
 	const gchar      *from;
 
+
 	if (lm_message_get_sub_type (m) != LM_MESSAGE_SUB_TYPE_GROUPCHAT) {
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
@@ -296,14 +326,11 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 	if (node) {
 		if (gossip_jid_get_resource (jid) == NULL) {
 			g_signal_emit_by_name (chatrooms->jabber,
-					       "chatroom-new-room-event",
+					       "chatroom-new-event",
 					       id, node->value);
 		} else {
-			gossip_time_t  timestamp;
-			GossipContact *contact;
+			gossip_time_t timestamp;
 
-			contact = jabber_chatrooms_get_contact (room, jid, 
-								NULL);
 			timestamp = gossip_jabber_get_message_timestamp (m);
 
 			message = gossip_message_new (GOSSIP_MESSAGE_TYPE_CHAT_ROOM,
@@ -312,6 +339,7 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 			timestamp = gossip_jabber_get_message_timestamp (m);
 			gossip_message_set_timestamp (message, timestamp);
 			
+			contact = jabber_chatrooms_get_contact (room, jid, NULL);
 			gossip_message_set_sender (message, contact);
 			gossip_message_set_body (message, node->value);
 			
@@ -325,9 +353,11 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 
 	node = lm_message_node_get_child (m->node, "subject");
 	if (node) {
+		contact = jabber_chatrooms_get_contact (room, jid, NULL);
+		
 		g_signal_emit_by_name (chatrooms->jabber,
-				       "chatroom-title-changed", 
-				       id, node->value);
+				       "chatroom-topic-changed", 
+				       id, contact, node->value);
 	}
 	
 	gossip_jid_unref (jid);
@@ -365,7 +395,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	
 	id = jabber_chatrooms_chatroom_get_id (room);
 
-	d(g_print ("Protocol Chatrooms: Presence from: %s\n", from));
+	DEBUG_MSG (("ProtocolChatrooms: Presence from: %s", from));
 
 	type = lm_message_get_sub_type (m);
 	switch (type) {
@@ -391,13 +421,13 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 
 		/* is contact new or updated */
 		if (new_contact || was_offline) {
-			d(g_print ("Protocol Chatrooms: ID[%d] Presence for new joining contact:'%s'\n",
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Presence for new joining contact:'%s'",
 				   id, gossip_jid_get_full (jid)));
 			g_signal_emit_by_name (chatrooms->jabber,
 					       "chatroom-contact-joined",
 					       id, contact);
 		} else {
-			d(g_print ("Protocol Chatrooms: ID[%d] Presence updated for contact:'%s'\n", 
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Presence updated for contact:'%s'", 
 				   id, gossip_jid_get_full (jid)));
 			g_signal_emit_by_name (chatrooms->jabber,
 					       "chatroom-contact-presence-updated", 
@@ -408,7 +438,7 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 	case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
 		contact = jabber_chatrooms_get_contact (room, jid, NULL);
 		if (contact) {
-			d(g_print ("Protocol Chatrooms: ID[%d] Contact left:'%s'\n",
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Contact left:'%s'",
 				   id, gossip_jid_get_full (jid)));
 			g_signal_emit_by_name (chatrooms->jabber, 
 					       "chatroom-contact-left",
@@ -418,38 +448,8 @@ jabber_chatrooms_presence_handler (LmMessageHandler      *handler,
 		}
 		break;
 
-#if 0
-	case LM_MESSAGE_SUB_TYPE_ERROR: 
-		if (room->callback) {
-			GossipChatroomStatus status;
-
-			status = gossip_chatroom_get_status (room->chatroom);
-
-			if (status == GOSSIP_CHATROOM_CONNECTING) {
-				/* malformed join group chat message is most likely */
-				gossip_chatroom_set_status (room->chatroom, 
-							    GOSSIP_CHATROOM_ERROR);
-
-				/* clean up timeout */
-				if (room->timeout_id) {
-					g_source_remove (room->timeout_id);
-					room->timeout_id = 0;
-				}
-
-				(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
-						  GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR, 
-						  id, room->user_data);
-
-				/* clean up callback */
-				room->callback = NULL;
-				room->user_data = NULL;
-			}
-		}
-		break;
-#endif
-
 	default:
-		d(g_print ("Protocol Chatrooms: Presence not handled for:'%s'\n",
+		DEBUG_MSG (("ProtocolChatrooms: Presence not handled for:'%s'",
 			   gossip_jid_get_full (jid)));
 		gossip_jid_unref (jid);
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
@@ -466,16 +466,10 @@ jabber_chatrooms_get_contact (JabberChatroom *room,
 	GSList        *l;
 	GossipContact *c;
 	const gchar   *id;
+	const gchar   *name;
 
 	id = gossip_jid_get_full (jid);
 
-	if (gossip_jid_equals (jid, room->jid)) {
-		if (new_contact) {
-			*new_contact = FALSE;
-		}
-		return room->own_contact;
-	}
-
 	for (l = room->contacts; l; l = l->next) {
 		c = GOSSIP_CONTACT (l->data);
 
@@ -483,6 +477,7 @@ jabber_chatrooms_get_contact (JabberChatroom *room,
 			if (new_contact) {
 				*new_contact = FALSE;
 			}
+
 			return c;
 		}
 	}
@@ -491,10 +486,15 @@ jabber_chatrooms_get_contact (JabberChatroom *room,
 		*new_contact = TRUE;
 	}
 
+	name = gossip_jid_get_resource (jid);
+	if (!name) {
+		name = id;
+	}
+
 	c = gossip_contact_new_full (GOSSIP_CONTACT_TYPE_CHATROOM, 
 				     gossip_contact_get_account (room->own_contact),
 				     id, 
-				     gossip_jid_get_resource (jid));
+				     name);
 	room->contacts = g_slist_prepend (room->contacts, c);
 
 	return c;
@@ -515,10 +515,10 @@ jabber_chatrooms_join_timeout_cb (JabberChatroom *room)
 	}
 
 	id = jabber_chatrooms_chatroom_get_id (room);
-	d(g_print ("Protocol Chatrooms: ID[%d] Join timed out (internally)\n", id));
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join timed out (internally)", id));
 
 	/* set chatroom status and error */
-	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_ERROR);
+	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_STATUS_ERROR);
 
 	last_error = gossip_chatroom_provider_join_result_as_str (GOSSIP_CHATROOM_JOIN_TIMED_OUT);
 	gossip_chatroom_set_last_error (room->chatroom, last_error);
@@ -527,7 +527,7 @@ jabber_chatrooms_join_timeout_cb (JabberChatroom *room)
 	chatrooms = room->chatrooms;
 		
 	if (room->callback != NULL) {
-		d(g_print ("Protocol Chatrooms: ID[%d] Calling back...\n", id));
+		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back... (timed out)", id));
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  GOSSIP_CHATROOM_JOIN_TIMED_OUT, 
 				  id, room->user_data);
@@ -570,10 +570,10 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 	if (existing_room) {
  		jabber_chatrooms_chatroom_unref (room); 
 
-		/* duplicate room already exists */
+		/* Duplicate room already exists. */
 		id = jabber_chatrooms_chatroom_get_id (existing_room);
 
-		d(g_print ("Protocol Chatrooms: ID[%d] Join chatroom:'%s', room already exists.\n", 
+		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s', room already exists.", 
 			   id,
 			   gossip_chatroom_get_room (chatroom)));
 
@@ -585,30 +585,30 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 		return id;
 	}
 
-	/* get real chatroom */
+	/* Get real chatroom. */
 	id = jabber_chatrooms_chatroom_get_id (room);
 
-        d(g_print ("Protocol Chatrooms: ID[%d] Join chatroom:'%s' on server:'%s'\n", 
+        DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s' on server:'%s'", 
 		   id,
 		   gossip_chatroom_get_room (chatroom),
 		   gossip_chatroom_get_server (chatroom)));
 
 
-	/* add timeout for server response */
+	/* Add timeout for server response. */
 	room->timeout_id = g_timeout_add (JOIN_TIMEOUT, 
 					  (GSourceFunc) jabber_chatrooms_join_timeout_cb, 
 					  room);
 
 	room->chatrooms = chatrooms;
 
-	/* set callback data */
+	/* Set callback data. */
         room->callback = callback;
         room->user_data = user_data;
 	
-	gossip_chatroom_set_status (chatroom, GOSSIP_CHATROOM_CONNECTING);
+	gossip_chatroom_set_status (chatroom, GOSSIP_CHATROOM_STATUS_JOINING);
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
-	/* compose message */
+	/* Compose message. */
 	m = lm_message_new_with_sub_type (gossip_jid_get_full (room->jid),
 					  LM_MESSAGE_TYPE_PRESENCE,
 					  LM_MESSAGE_SUB_TYPE_AVAILABLE);
@@ -620,7 +620,6 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 			     room->jid, 
 			     jabber_chatrooms_chatroom_ref (room));
 
-	/* release the local ref */
 	jabber_chatrooms_chatroom_unref (room); 
 
         show = gossip_jabber_presence_state_to_str (chatrooms->presence);
@@ -629,11 +628,11 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
                 lm_message_node_add_child (m->node, "show", show);
 	}
 
-        id_str = g_strdup_printf (JOIN_MSG_ID_PREFIX "%d", id);
+        id_str = g_strdup_printf ("muc_join_%d", id);
         lm_message_node_set_attribute (m->node, "id", id_str);
         g_free (id_str);
 
-	/* send message */
+	/* Send message. */
 	room->connection = lm_connection_ref (chatrooms->connection);
         room->join_handler = lm_message_handler_new ((LmHandleMessageFunction) 
 						     jabber_chatrooms_join_cb,
@@ -644,8 +643,8 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 						LM_MESSAGE_TYPE_PRESENCE,
 						LM_HANDLER_PRIORITY_FIRST);
         
-	/* some servers don't honor the id so we don't get a reply and
-	   are waiting forever */
+	/* Some servers don't honor the id so we don't get a reply and
+	   are waiting forever. */
  	lm_connection_send (chatrooms->connection, m,  NULL);
 
 	lm_message_unref (m);
@@ -663,8 +662,13 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 	GossipChatroomJoinResult  result;
 	GossipChatroomStatus      status;
 	GossipChatroomId          id;
+	GossipChatroomId          id_found;
 	LmMessageSubType          type;
 	LmMessageNode            *node = NULL;
+	const gchar              *from;
+	GossipJID                *jid;
+	JabberChatroom           *room_found;
+	gboolean                  room_match = FALSE;
 
 	if (!room || !room->join_handler) {
  		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS; 
@@ -672,6 +676,26 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 
 	chatrooms = room->chatrooms;
 
+	/* get room id */
+	id = jabber_chatrooms_chatroom_get_id (room);
+
+	from = lm_message_node_get_attribute (m->node, "from");
+	jid = gossip_jid_new (from);
+
+	room_found = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
+	gossip_jid_unref (jid);
+
+	if (room_found) {
+		id_found = jabber_chatrooms_chatroom_get_id (room_found);
+		if (id == id_found) {
+			room_match = TRUE;
+		}
+	} 
+
+	if (!room_match) {
+ 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS; 
+	}
+
 	/* clean up the join timeout */
 	if (room->timeout_id) {
 		g_source_remove (room->timeout_id);
@@ -684,55 +708,54 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 		room->join_handler = NULL;
 	}
 
-	/* get room id */
-	id = jabber_chatrooms_chatroom_get_id (room);
-
         /* check for error */
-	type = lm_message_get_sub_type (m);
+ 	type = lm_message_get_sub_type (m);
 	if (type == LM_MESSAGE_SUB_TYPE_ERROR) {
 		node = lm_message_node_get_child (m->node, "error");
 	}
 	
 	if (node) {
 		const gchar *str;
-		gint         code;
-		
+		gint         code;		
 		str = lm_message_node_get_attribute (node, "code");
 		code = str ? atoi (str) : 0;
 		
 		switch (code) {
-		case 409: {
-			/* conflicting nickname */
-			d(g_print ("Protocol Chatrooms: ID[%d] Conflicting nickname\n", id));
-			result = GOSSIP_CHATROOM_JOIN_NICK_IN_USE;
-			break;
-		}
-			
-		case 502: {
-			/* unresolved hostname */
-			d(g_print ("Protocol Chatrooms: ID[%d] Unable to resolve hostname\n", id));
+		case 404: 
+ 			/* conflicting nickname */
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Chatroom not found", id));
 			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST;
 			break;
-		}
 			
-		case 504: {
+		case 409: 
+ 			/* conflicting nickname */
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Conflicting nickname", id));
+			result = GOSSIP_CHATROOM_JOIN_NICK_IN_USE;
+			break;
+			
+		case 502: 
+			/* unresolved hostname */
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Unable to resolve hostname", id));
+			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_HOST;
+			break;
+			
+		case 504: 
 			/* remote server timeout */
-			d(g_print ("Protocol Chatrooms: ID[%d] Join timed out\n", id));
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join timed out", id));
 			result = GOSSIP_CHATROOM_JOIN_TIMED_OUT;
 			break;
-		}
 			
 		default:
-			d(g_print ("Protocol Chatrooms: ID[%d] Unhandled presence error:%d\n", id, code));
+			DEBUG_MSG (("ProtocolChatrooms: ID[%d] Unhandled presence error:%d", id, code));
 			result = GOSSIP_CHATROOM_JOIN_UNKNOWN_ERROR;
 			break;
 		}
 
 		/* set room state */
-		status = GOSSIP_CHATROOM_ERROR;
+		status = GOSSIP_CHATROOM_STATUS_ERROR;
 	} else {
 		result = GOSSIP_CHATROOM_JOIN_OK;
-		status = GOSSIP_CHATROOM_OPEN;
+		status = GOSSIP_CHATROOM_STATUS_ACTIVE;
 	}
 
 	gossip_chatroom_set_status (room->chatroom, status);
@@ -740,26 +763,25 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 					gossip_chatroom_provider_join_result_as_str (result));
 
 	if (room->callback != NULL) {
-		d(g_print ("Protocol Chatrooms: ID[%d] Calling back...\n", id));
+		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back...", id));
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  result, id, room->user_data);
 	}
 
-	/* clean up callback data */
+	/* Clean up callback data */
 	room->callback = NULL;
 	room->user_data = NULL;
 
-	/* articulate own contact presence so we appear in group chat */
+	/* Articulate own contact presence so we appear in group chat */
 	if (result == GOSSIP_CHATROOM_JOIN_OK) {
 		gossip_contact_add_presence (room->own_contact, 
 					     chatrooms->presence);
 		
 		g_signal_emit_by_name (chatrooms->jabber,
-				       "chatroom-contact-joined",
-				       id,
-				       room->own_contact);
+				       "chatroom-joined",
+				       id);
 	} else {
-		/* clean up */
+		/* Clean up */
 		g_hash_table_remove (chatrooms->room_id_hash, 
 				     GINT_TO_POINTER (id));
 		g_hash_table_remove (chatrooms->room_jid_hash, 
@@ -780,11 +802,11 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 	room = g_hash_table_lookup (chatrooms->room_id_hash, 
 				    GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	d(g_print ("Protocol Chatrooms: ID[%d] Cancel joining room\n", id));
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Cancel joining room", id));
 
 	if (room->timeout_id) {
 		g_source_remove (room->timeout_id);
@@ -797,11 +819,11 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 	}
 
 	
-	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_CLOSED);
+	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_STATUS_INACTIVE);
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
 	if (room->callback != NULL) {
-		d(g_print ("Protocol Chatrooms: ID[%d] Calling back...\n", id));
+		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Calling back...", id));
 		(room->callback) (GOSSIP_CHATROOM_PROVIDER (chatrooms->jabber),
 				  GOSSIP_CHATROOM_JOIN_CANCELED, id, room->user_data);
 	}
@@ -830,11 +852,11 @@ gossip_jabber_chatrooms_send (GossipJabberChatrooms *chatrooms,
 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
 						       GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 	
-	d(g_print ("Protocol Chatrooms: ID[%d] Send message\n", id));
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Send message", id));
 	
 	m = lm_message_new_with_sub_type (gossip_jid_get_without_resource (room->jid),
 					  LM_MESSAGE_TYPE_MESSAGE,
@@ -860,11 +882,11 @@ gossip_jabber_chatrooms_change_topic (GossipJabberChatrooms *chatrooms,
 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
 						       GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	d(g_print ("Protocol Chatrooms: ID[%d] Change topic to:'%s'\n", 
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Change topic to:'%s'", 
 		   id, new_topic));
 	
 	without_resource = gossip_jid_get_without_resource (room->jid);
@@ -893,11 +915,11 @@ gossip_jabber_chatrooms_change_nick (GossipJabberChatrooms *chatrooms,
 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
 						       GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	d(g_print ("Protocol Chatrooms: ID[%d] Change chatroom nick to:'%s'\n", 
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Change chatroom nick to:'%s'", 
 		   id, new_nick));
 
 	gossip_jid_set_resource (room->jid, new_nick);
@@ -921,11 +943,11 @@ gossip_jabber_chatrooms_leave (GossipJabberChatrooms *chatrooms,
 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
 						       GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_CLOSED);
+	gossip_chatroom_set_status (room->chatroom, GOSSIP_CHATROOM_STATUS_INACTIVE);
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
 	m = lm_message_new_with_sub_type (gossip_jid_get_full (room->jid),
@@ -940,15 +962,15 @@ gossip_jabber_chatrooms_leave (GossipJabberChatrooms *chatrooms,
 	g_hash_table_remove (chatrooms->room_jid_hash,
 			     room->jid);
 
-	d(g_print ("Protocol Chatrooms: ID[%d] Leaving room, ref count is %d\n", 
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Leaving room, ref count is %d", 
 		   id, room->ref_count - 1));
 	
 	jabber_chatrooms_chatroom_unref (room);
 }
 
-const gchar *
-gossip_jabber_chatrooms_get_room_name (GossipJabberChatrooms *chatrooms,
-				       GossipChatroomId       id)
+GossipChatroom *
+gossip_jabber_chatrooms_find (GossipJabberChatrooms *chatrooms,
+			      GossipChatroomId       id)
 {
 	JabberChatroom *room;
 
@@ -957,48 +979,52 @@ gossip_jabber_chatrooms_get_room_name (GossipJabberChatrooms *chatrooms,
 	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash,
 						      GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
 		return NULL;
 	}
 
-	return gossip_chatroom_get_name (room->chatroom);
+	return room->chatroom;
 }
 
 void 
 gossip_jabber_chatrooms_invite (GossipJabberChatrooms *chatrooms,
 				GossipChatroomId       id,
-				const gchar           *contact_id,
-				const gchar           *invite)
+				GossipContact         *contact,
+				const gchar           *reason)
 {
 	LmMessage      *m;
-	LmMessageNode  *n;
+	LmMessageNode  *parent;
+	LmMessageNode  *node;
 	JabberChatroom *room;
 
 	g_return_if_fail (chatrooms != NULL);
-	g_return_if_fail (contact_id != NULL);
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	room = (JabberChatroom*) g_hash_table_lookup (chatrooms->room_id_hash, 
+						      GINT_TO_POINTER (id));
 
-	room = (JabberChatroom *) g_hash_table_lookup (chatrooms->room_id_hash, 
-						       GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("Protocol Chatrooms: Unknown chatroom id: %d", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
-	d(g_print ("Protocol Chatrooms: ID[%d] Inviting contact:'%s'\n", 
-		   id, contact_id));
+	DEBUG_MSG (("ProtocolChatrooms: ID[%d] Invitation to contact:'%s' from:'%s'", 
+		    id, 
+		    gossip_contact_get_id (contact), 
+		    gossip_contact_get_id (room->own_contact)));
 
-	m = lm_message_new (contact_id,
+	m = lm_message_new (gossip_jid_get_without_resource (room->jid),
 			    LM_MESSAGE_TYPE_MESSAGE);
-
-	if (invite) {
-		lm_message_node_add_child (m->node, "body", invite); 
-	}
-	
-	n = lm_message_node_add_child (m->node, "x", NULL);
-	lm_message_node_set_attributes (n, 
-					"jid", gossip_jid_get_without_resource (room->jid),
-					"xmlns", "jabber:x:conference",
+	lm_message_node_set_attributes (m->node, 
+					"from", gossip_contact_get_id (room->own_contact), 
 					NULL);
+	
+	parent = lm_message_node_add_child (m->node, "x", NULL);
+	lm_message_node_set_attributes (parent, "xmlns", XMPP_MUC_USER_XMLNS, NULL);
+
+	node = lm_message_node_add_child (parent, "invite", NULL);
+	lm_message_node_set_attributes (node, "to", gossip_contact_get_id (contact), NULL);
+
+	lm_message_node_add_child (node, "reason", reason);
 
 	lm_connection_send (chatrooms->connection, m, NULL);
 	lm_message_unref (m);
@@ -1007,28 +1033,38 @@ gossip_jabber_chatrooms_invite (GossipJabberChatrooms *chatrooms,
 void 
 gossip_jabber_chatrooms_invite_accept (GossipJabberChatrooms *chatrooms,
 				       GossipChatroomJoinCb   callback,
-				       const gchar           *nickname,
-				       const gchar           *invite_id)
+				       GossipChatroomInvite  *invite,
+				       const gchar           *nickname)
 {
 	GossipChatroom *chatroom;
+	GossipContact  *contact;
+	GossipAccount  *account;
 	gchar          *room = NULL;
+	const gchar    *id;
 	const gchar    *server;
 
-	g_return_if_fail (invite_id != NULL);
+	g_return_if_fail (chatrooms != NULL);
+	g_return_if_fail (invite != NULL);
 	g_return_if_fail (callback != NULL);
 
-	server = strstr (invite_id, "@");
+	id = gossip_chatroom_invite_get_id (invite);
+	contact = gossip_chatroom_invite_get_invitor (invite);
+
+	server = strstr (id, "@");
 
 	g_return_if_fail (server != NULL);
 	g_return_if_fail (nickname != NULL);
 	
 	if (server) {
-		room = g_strndup (invite_id, server - invite_id);
+		room = g_strndup (id, server - id);
 		server++;
 	}
 
+ 	account = gossip_contact_get_account (contact);
+
 	chatroom = g_object_new (GOSSIP_TYPE_CHATROOM, 
 				 "type", GOSSIP_CHATROOM_TYPE_NORMAL,
+				 "account", account,
 				 "server", server,
 				 "name", room,
 				 "room", room,
@@ -1044,6 +1080,44 @@ gossip_jabber_chatrooms_invite_accept (GossipJabberChatrooms *chatrooms,
 	g_free (room);
 }
 
+void 
+gossip_jabber_chatrooms_invite_decline (GossipJabberChatrooms *chatrooms,
+					GossipChatroomInvite  *invite,
+					const gchar           *reason)
+{
+	LmMessage     *m;
+	LmMessageNode *n;
+	GossipContact *own_contact;
+	GossipContact *contact;
+	const gchar   *id;
+
+	g_return_if_fail (chatrooms != NULL);
+	g_return_if_fail (invite != NULL);
+
+	own_contact = gossip_jabber_get_own_contact (chatrooms->jabber);
+	contact = gossip_chatroom_invite_get_invitor (invite);
+	id = gossip_chatroom_invite_get_id (invite);
+
+	DEBUG_MSG (("ProtocolChatrooms: Invitation decline to:'%s' into room:'%s'", 
+		    gossip_contact_get_id (contact), id));
+
+	m = lm_message_new (id, LM_MESSAGE_TYPE_MESSAGE);
+	lm_message_node_set_attributes (m->node, 
+					"from", gossip_contact_get_id (own_contact), 
+					NULL);
+
+	n = lm_message_node_add_child (m->node, "x", NULL);
+	lm_message_node_set_attributes (n, "xmlns", XMPP_MUC_USER_XMLNS, NULL);
+
+	n = lm_message_node_add_child (n, "decline", NULL);
+	lm_message_node_set_attributes (n, "to", gossip_contact_get_id (contact), NULL);
+
+	n = lm_message_node_add_child (n, "reason", reason);
+
+	lm_connection_send (chatrooms->connection, m, NULL);
+	lm_message_unref (m);
+}
+
 GList * 
 gossip_jabber_chatrooms_get_rooms (GossipJabberChatrooms *chatrooms)
 {
@@ -1133,11 +1207,3 @@ gossip_jabber_chatrooms_get_jid_is_chatroom (GossipJabberChatrooms *chatrooms,
 
 	return ret_val;
 }
-
-GossipContact * 
-gossip_jabber_chatrooms_get_contact (GossipJabberChatrooms *chatrooms,
-				     LmMessage             *message,
-				     gint                  *chat_id)
-{
-	return NULL;
-}
