@@ -26,7 +26,12 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
+
+#ifdef USE_GNOMEVFS_FOR_URL
+#include <libgnomevfs/gnome-vfs.h>
+#else
 #include <libgnome/gnome-url.h>
+#endif
 
 #include <libgossip/gossip-time.h>
 #include <libgossip/gossip-utils.h>
@@ -37,7 +42,9 @@
 #include "gossip-theme-manager.h"
 #include "gossip-app.h"
 
-#define DEBUG_MSG(x) 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+
+#define DEBUG_MSG(x)
 /* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 /* Number of seconds between timestamps when using normal mode, 5 minutes. */
@@ -46,7 +53,6 @@
 /* Maximum lines in any chat buffer, see bug #141292. */
 #define MAX_LINES 1000
 
-
 typedef enum {
 	BLOCK_TYPE_NONE,
 	BLOCK_TYPE_SELF,
@@ -56,7 +62,6 @@ typedef enum {
 	BLOCK_TYPE_INVITE
 } BlockType;
 
-
 struct _GossipChatViewPriv {
 	GtkTextBuffer *buffer;
 
@@ -64,20 +69,20 @@ struct _GossipChatViewPriv {
 	time_t         last_timestamp;
 	BlockType      last_block_type;
 
+	gboolean       allow_scrolling;
+
 	/* This is for the group chat so we know if the "other" last contact
 	 * changed, so we know whether to insert a header or not.
 	 */
 	GossipContact *last_contact;
 };
 
-
 typedef struct {
 	GossipSmiley  smiley;
 	gchar        *pattern;
 	gint          index;
 } GossipSmileyPattern;
 
-
 static GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_NORMAL,       ":)",  0 },
 	{ GOSSIP_SMILEY_WINK,         ";)",  0 },
@@ -168,10 +173,8 @@ static GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_SICK,         ")o+", 0 }
 };
 
-
 static gint num_smileys = G_N_ELEMENTS (smileys);
 
-
 static void       gossip_chat_view_class_init          (GossipChatViewClass      *klass);
 static void       gossip_chat_view_init                (GossipChatView           *view);
 static void       chat_view_finalize                   (GObject                  *object);
@@ -204,10 +207,11 @@ static void       chat_view_clear_view_cb              (GtkMenuItem
 static void       chat_view_insert_text_with_emoticons (GtkTextBuffer            *buf,
 							GtkTextIter              *iter,
 							const gchar              *str);
-static GdkPixbuf *chat_view_get_smiley                 (GossipSmiley              smiley);
 static gboolean   chat_view_is_scrolled_down           (GossipChatView           *view);
 static void       chat_view_invite_accept_cb           (GtkWidget                *button,
 							gpointer                  user_data);
+static void       chat_view_invite_decline_cb          (GtkWidget                *button,
+							gpointer                  user_data);
 static void       chat_view_invite_join_cb             (GossipChatroomProvider   *provider,
 							GossipChatroomJoinResult  result,
 							gint                      id,
@@ -241,10 +245,8 @@ static void       chat_view_append_fancy_message       (GossipChatView
 							GossipContact            *my_contact,
 							gboolean                  from_self);
 
-
 G_DEFINE_TYPE (GossipChatView, gossip_chat_view, GTK_TYPE_TEXT_VIEW);
 
-
 static void
 gossip_chat_view_class_init (GossipChatViewClass *klass)
 {
@@ -254,6 +256,8 @@ gossip_chat_view_class_init (GossipChatViewClass *klass)
 	object_class->finalize = chat_view_finalize;
 	widget_class->size_allocate = chat_view_size_allocate;
 	widget_class->drag_motion = chat_view_drag_motion;
+
+	g_type_class_add_private (object_class, sizeof (GossipChatViewPriv));
 }
 
 static void
@@ -261,13 +265,14 @@ gossip_chat_view_init (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
 
-	priv = g_new0 (GossipChatViewPriv, 1);
-	view->priv = priv;
+	priv = GET_PRIV (view);
+
+	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
 	priv->last_block_type = BLOCK_TYPE_NONE;
 	priv->last_timestamp = 0;
 
-	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	priv->allow_scrolling = TRUE;
 
 	g_object_set (view,
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
@@ -294,13 +299,6 @@ gossip_chat_view_init (GossipChatView *view)
 static void
 chat_view_finalize (GObject *object)
 {
-	GossipChatView     *view = GOSSIP_CHAT_VIEW (object);
-	GossipChatViewPriv *priv;
-
-	priv = view->priv;
-
-	g_free (priv);
-
 	G_OBJECT_CLASS (gossip_chat_view_parent_class)->finalize (object);
 }
 
@@ -339,12 +337,17 @@ chat_view_setup_tags (GossipChatView *view)
 	GossipChatViewPriv *priv;
 	GtkTextTag         *tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	gtk_text_buffer_create_tag (priv->buffer,
 				    "cut",
 				    NULL);
 
+	gtk_text_buffer_create_tag (priv->buffer,
+				    "highlight",
+				    "background", "yellow",
+				    NULL);
+
 	tag = gtk_text_buffer_create_tag (priv->buffer,
 					  "link",
 					  NULL);
@@ -373,8 +376,25 @@ chat_view_populate_popup (GossipChatView *view,
 	GtkWidget          *item;
 	gchar              *str = NULL;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
+	/* Clear menu item */
+	if (gtk_text_buffer_get_char_count (priv->buffer) > 0) {
+		item = gtk_menu_item_new ();
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
+		
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_widget_show (item);
+		
+		g_signal_connect (item,
+				  "activate",
+				  G_CALLBACK (chat_view_clear_view_cb),
+				  view);
+	}
+
+	/* Link context menu items */
 	table = gtk_text_buffer_get_tag_table (priv->buffer);
 	tag = gtk_text_tag_table_lookup (table, "link");
 
@@ -391,32 +411,24 @@ chat_view_populate_popup (GossipChatView *view,
 
 	if (gtk_text_iter_backward_to_tag_toggle (&start, tag) &&
 	    gtk_text_iter_forward_to_tag_toggle (&end, tag)) {
-
 		str = gtk_text_buffer_get_text (priv->buffer,
 						&start, &end, FALSE);
 	}
 
-	item = gtk_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	item = gtk_menu_item_new_with_mnemonic (_("C_lear"));
-	g_signal_connect (item,
-			  "activate",
-			  G_CALLBACK (chat_view_clear_view_cb),
-			  view);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	if (!str || strlen (str) == 0) {
+	if (!str || strlen (str) < 1) {
+		g_free (str);
 		return;
 	}
 
-	/* Set data just to get the string freed when not needed. */
+	/* NOTE: Set data just to get the string freed when not needed. */
 	g_object_set_data_full (G_OBJECT (menu),
 				"url", str,
 				(GDestroyNotify) g_free);
 
+	item = gtk_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
 	item = gtk_menu_item_new_with_mnemonic (_("_Copy Link Address"));
 	g_signal_connect (item,
 			  "activate",
@@ -517,23 +529,16 @@ chat_view_url_event_cb (GtkTextTag    *tag,
 static void
 chat_view_open_address (const gchar *url)
 {
-	if (!url || strlen (url) == 0) {
-		return;
+#ifdef USE_GNOMEVFS_FOR_URL
+	GnomeVFSResult result;
+
+	result = gnome_vfs_url_show (url);
+	if (result == GNOME_VFS_OK) {
+		g_warning ("Couldn't show URL:'%s'", url);
 	}
-
-	/* gnome_url_show doesn't work when there's no protocol, so we might
-	 * need to add one.
-	 */
-	if (strstr (url, "://") == NULL) {
-		gchar *tmp;
-
-		tmp = g_strconcat ("http://", url, NULL);
-		gnome_url_show (tmp, NULL);
-		g_free (tmp);
-		return;
-	}
-
+#else
 	gnome_url_show (url, NULL);
+#endif
 }
 
 static void
@@ -583,6 +588,10 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 	}
 
 	while (*str) {
+		GdkPixbuf   *pixbuf;
+		gint         len;
+		const gchar *start;
+
 		for (i = 0; i < num_smileys; i++) {
 			smileys[i].index = 0;
 		}
@@ -591,6 +600,7 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 		submatch = -1;
 		p = str;
 		prev_c = 0;
+
 		while (*p) {
 			c = g_utf8_get_char (p);
 
@@ -621,57 +631,27 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 			p = g_utf8_next_char (p);
 		}
 
-		if (match != -1) {
-			GdkPixbuf   *pixbuf;
-			gint         len;
-			const gchar *start;
-
-			start = p - strlen (smileys[match].pattern);
-
-			if (start > str) {
-				len = start - str;
-				gtk_text_buffer_insert (buf, iter, str, len);
-			}
-
-			pixbuf = chat_view_get_smiley (smileys[match].smiley);
-			gtk_text_buffer_insert_pixbuf (buf, iter, pixbuf);
-
-			gtk_text_buffer_insert (buf, iter, " ", 1);
-		} else {
+		if (match == -1) {
 			gtk_text_buffer_insert (buf, iter, str, -1);
 			return;
 		}
 
+		start = p - strlen (smileys[match].pattern);
+		
+		if (start > str) {
+			len = start - str;
+			gtk_text_buffer_insert (buf, iter, str, len);
+		}
+		
+		pixbuf = gossip_chat_view_get_smiley_image (smileys[match].smiley);
+		gtk_text_buffer_insert_pixbuf (buf, iter, pixbuf);
+		
+		gtk_text_buffer_insert (buf, iter, " ", 1);
+
 		str = g_utf8_find_next_char (p, NULL);
 	}
 }
 
-static GdkPixbuf *
-chat_view_get_smiley (GossipSmiley smiley)
-{
-	static GdkPixbuf *pixbufs[NUM_SMILEYS];
-	static gboolean   inited = FALSE;
-
-	if (!inited) {
-		gint i;
-
-		for (i = 0; i < NUM_SMILEYS; i++) {
-			pixbufs[i] = gossip_pixbuf_from_smiley (
-				i, GTK_ICON_SIZE_MENU);
-		}
-
-		inited = TRUE;
-	}
-
-	return pixbufs[smiley];
-}
-
-GossipChatView *
-gossip_chat_view_new (void)
-{
-	return g_object_new (GOSSIP_TYPE_CHAT_VIEW, NULL);
-}
-
 static gboolean
 chat_view_is_scrolled_down (GossipChatView *view)
 {
@@ -691,29 +671,6 @@ chat_view_is_scrolled_down (GossipChatView *view)
 	return TRUE;
 }
 
-void
-gossip_chat_view_scroll_down (GossipChatView *view)
-{
-	GtkTextBuffer *buffer;
-	GtkTextIter    iter;
-	GtkTextMark   *mark;
-
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-
-	gtk_text_buffer_get_end_iter (buffer, &iter);
-	mark = gtk_text_buffer_create_mark (buffer,
-					    NULL,
-					    &iter,
-					    FALSE);
-
-	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
-				      mark,
-				      0.0,
-				      FALSE,
-				      0,
-				      0);
-}
-
 static void
 chat_view_maybe_trim_buffer (GossipChatView *view)
 {
@@ -724,7 +681,7 @@ chat_view_maybe_trim_buffer (GossipChatView *view)
 	GtkTextTagTable    *table;
 	GtkTextTag         *tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	gtk_text_buffer_get_end_iter (priv->buffer, &bottom);
 	line = gtk_text_iter_get_line (&bottom);
@@ -758,99 +715,6 @@ chat_view_maybe_trim_buffer (GossipChatView *view)
 	}
 }
 
-void
-gossip_chat_view_append_invite (GossipChatView *view,
-				GossipMessage  *message)
-{
-	GossipChatViewPriv *priv;
-	GossipContact      *sender;
-	const gchar        *invite;
-	const gchar        *body;
-	GtkTextChildAnchor *anchor;
-	GtkTextIter         iter;
-	GtkWidget          *widget;
-	const gchar        *used_msg;
-	const gchar        *used_invite;
-	gchar              *str;
-	gboolean            bottom;
-	const gchar        *tag;
-
-	priv = view->priv;
-
-	if (priv->irc_style) {
-		tag = "irc-invite";
-	} else {
-		tag = "fancy-invite";
-	}
-	
-	bottom = chat_view_is_scrolled_down (view);
-
-	sender = gossip_message_get_sender (message);
-	invite = gossip_message_get_invite (message);
-	body = gossip_message_get_body (message);
-		
-	chat_view_maybe_append_date_and_time (view, message);
-
-	if (body && body[0]) {
-		used_msg = body;
-	} else {
-		used_msg = _("You have been invited to join a chat conference.");
-	}
-
-	/* Don't include the invite in the chat window if it is part of the
-	 * actual request - some chat clients send this and it looks weird
-	 * repeated.
-	 */
-	if (strstr (body, invite)) {
-		used_invite = NULL;
-	} else {
-		used_invite = invite;
-	}
-
-	str = g_strdup_printf ("\n%s\n%s%s%s%s\n",
-			       used_msg,
-			       used_invite ? "(" : "",
-			       used_invite ? used_invite : "",
-			       used_invite ? ")" : "",
-			       used_invite ? "\n" : "");
-	chat_view_append_text (view, str, tag);
-	g_free (str);
-
-	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
-	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
-
-	widget = gtk_button_new_with_label (_("Accept"));
-	g_object_set_data_full (G_OBJECT (widget), "invite",
-				g_strdup (invite), g_free);
- 	g_object_set_data_full (G_OBJECT (widget), "contact",
- 				g_object_ref (sender), g_object_unref);
-
-	g_signal_connect (widget,
-			  "clicked",
-			  G_CALLBACK (chat_view_invite_accept_cb),
-			  NULL);
-
-	gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view),
-					   widget,
-					   anchor);
-
-	gtk_widget_show_all (widget);
-
-	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
-	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
-						  &iter,
-						  "\n\n",
-						  2,
-						  tag,
-						  NULL);
-
-	if (bottom) {
-		gossip_chat_view_scroll_down (view);
-	}
-
-	priv->last_block_type = BLOCK_TYPE_INVITE;
-}
-
 static void
 chat_view_maybe_append_date_and_time (GossipChatView *view,
 				      GossipMessage  *msg)
@@ -863,7 +727,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	gboolean            append_date, append_time;
 	GString            *str;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	if (priv->irc_style) {
 		tag = "irc-time";
@@ -876,8 +740,12 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 	}
 
 	str = g_string_new (NULL);
+
+	timestamp = 0;
+	if (msg) {
+		timestamp = gossip_message_get_timestamp (msg);
+	}
 	
-	timestamp = gossip_message_get_timestamp (msg);
 	if (timestamp <= 0) {
 		timestamp = gossip_time_get_current ();
 	}
@@ -895,7 +763,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 		append_date = TRUE;
 		append_time = TRUE;
 	}
-
+	
 	if (priv->last_timestamp + TIMESTAMP_INTERVAL < timestamp) {
 		append_time = TRUE;
 	}
@@ -913,7 +781,7 @@ chat_view_maybe_append_date_and_time (GossipChatView *view,
 		g_string_append (str, buf);
 
 		if (append_time) {
-			g_string_append (str, " - ");
+			g_string_append (str, ", ");
 		}
 	}
 
@@ -952,7 +820,7 @@ chat_view_append_spacing (GossipChatView *view)
 	const gchar        *tag;
 	GtkTextIter         iter;
 	
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	if (priv->irc_style) {
 		tag = "irc-spacing";
@@ -983,7 +851,7 @@ chat_view_append_text (GossipChatView *view,
 	GArray             *start, *end;
 	const gchar        *link_tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	if (priv->irc_style) {
 		link_tag = "irc-link";
@@ -997,7 +865,8 @@ chat_view_append_text (GossipChatView *view,
 	start = g_array_new (FALSE, FALSE, sizeof (gint));
 	end = g_array_new (FALSE, FALSE, sizeof (gint));
 
-	num_matches = gossip_utils_url_regex_match (body, start, end);
+	num_matches = gossip_utils_regex_match (GOSSIP_REGEX_ALL, 
+						body, start, end);
 
 	if (num_matches == 0) {
 		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
@@ -1081,7 +950,7 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 	const gchar        *tag;
 	const gchar        *line_tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	contact = gossip_message_get_sender (msg);
 	
@@ -1166,7 +1035,7 @@ chat_view_append_irc_action (GossipChatView *view,
 	gchar              *tmp;
 	const gchar        *tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	DEBUG_MSG (("ChatView: Add IRC action"));
 
@@ -1220,7 +1089,7 @@ chat_view_append_fancy_action (GossipChatView *view,
 	const gchar        *tag;
 	const gchar        *line_tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	DEBUG_MSG (("ChatView: Add fancy action"));
 
@@ -1269,7 +1138,7 @@ chat_view_append_irc_message (GossipChatView *view,
 	GtkTextIter         iter;
 	gchar              *tmp;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	DEBUG_MSG (("ChatView: Add IRC message"));
 
@@ -1327,7 +1196,7 @@ chat_view_append_fancy_message (GossipChatView *view,
 	const gchar        *body;
 	const gchar        *tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 	
 	if (from_self) {
 		tag = "fancy-body-self";
@@ -1341,6 +1210,95 @@ chat_view_append_fancy_message (GossipChatView *view,
 	chat_view_append_text (view, body, tag);
 }
 
+static void
+chat_view_invite_accept_cb (GtkWidget *button,
+			    gpointer   user_data)
+{
+	GossipSession          *session;
+	GossipAccount          *account;
+	GossipContact          *contact;
+	GossipChatroomProvider *provider;
+	GossipChatroomInvite   *invite;
+	GtkWidget              *other_button;
+	const gchar            *nickname;
+	const gchar            *reason;
+
+	invite = g_object_get_data (G_OBJECT (button), "invite");
+	other_button = g_object_get_data (G_OBJECT (button), "other_button");
+
+	gtk_widget_set_sensitive (button, FALSE);
+	gtk_widget_set_sensitive (other_button, FALSE);
+
+	contact = gossip_chatroom_invite_get_invitor (invite);
+	reason = gossip_chatroom_invite_get_reason (invite);
+
+	session = gossip_app_get_session ();
+	account = gossip_contact_get_account (contact);
+	provider = gossip_session_get_chatroom_provider (session, account);
+	nickname = gossip_session_get_nickname (session, account);
+
+	gossip_chatroom_provider_invite_accept (provider,
+						chat_view_invite_join_cb,
+						invite,
+						nickname);
+}
+
+static void
+chat_view_invite_decline_cb (GtkWidget *button,
+			     gpointer   user_data)
+{
+	GossipSession          *session;
+	GossipAccount          *account;
+	GossipContact          *contact;
+	GossipChatroomProvider *provider;
+	GossipChatroomInvite   *invite;
+	GtkWidget              *other_button;
+	const gchar            *reason;
+
+	invite = g_object_get_data (G_OBJECT (button), "invite");
+	other_button = g_object_get_data (G_OBJECT (button), "other_button");
+
+	gtk_widget_set_sensitive (button, FALSE);
+	gtk_widget_set_sensitive (other_button, FALSE);
+
+	contact = gossip_chatroom_invite_get_invitor (invite);
+
+	session = gossip_app_get_session ();
+	account = gossip_contact_get_account (contact);
+	provider = gossip_session_get_chatroom_provider (session, account);
+
+	reason = _("Your invitation has been declined");
+	gossip_chatroom_provider_invite_decline (provider, invite, reason);
+}
+
+static void
+chat_view_invite_join_cb (GossipChatroomProvider   *provider,
+			  GossipChatroomJoinResult  result,
+			  gint                      id,
+			  gpointer                  user_data)
+{
+	gossip_group_chat_new (provider, id);
+}
+
+static void
+theme_manager_theme_changed_cb (GossipThemeManager *manager,
+				GossipChatView     *view)
+{
+	GossipChatViewPriv *priv;
+	
+	priv = GET_PRIV (view);
+	
+	priv->last_block_type = BLOCK_TYPE_NONE;
+	
+	gossip_theme_manager_apply (manager, view);
+}
+
+GossipChatView *
+gossip_chat_view_new (void)
+{
+	return g_object_new (GOSSIP_TYPE_CHAT_VIEW, NULL);
+}
+
 /* The name is optional, if NULL, the sender for msg is used. */
 void
 gossip_chat_view_append_message_from_self (GossipChatView *view,
@@ -1351,7 +1309,7 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 	const gchar        *body;
 	gboolean            scroll_down;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	body = gossip_message_get_body (msg);
 	if (!body) {
@@ -1407,7 +1365,7 @@ gossip_chat_view_append_message_from_other (GossipChatView *view,
 	const gchar        *body;
 	gboolean            scroll_down;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	body = gossip_message_get_body (msg);
 	if (!body) {
@@ -1460,12 +1418,10 @@ gossip_chat_view_append_event (GossipChatView *view,
 	GossipChatViewPriv *priv;
 	gboolean            bottom;
 	GtkTextIter         iter;
-	time_t              timestamp;
-	gchar              *stamp;
 	gchar              *msg;
 	const gchar        *tag;
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 	
 	bottom = chat_view_is_scrolled_down (view);
 
@@ -1473,20 +1429,19 @@ gossip_chat_view_append_event (GossipChatView *view,
 
 	if (priv->irc_style) {
 		tag = "irc-event";
+		msg = g_strdup_printf (" - %s\n", str);
 	} else {
 		tag = "fancy-event";
+		msg = g_strdup_printf (" - %s\n", str);
 	}
 	
 	if (priv->last_block_type != BLOCK_TYPE_EVENT) {
 		chat_view_append_spacing (view);
 	}
-		
-	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 
-	timestamp = gossip_time_get_current ();
-	stamp = gossip_time_to_timestamp (-1);
-	msg = g_strdup_printf ("- %s - %s -\n", str, stamp);
-	g_free (stamp);
+	chat_view_maybe_append_date_and_time (view, NULL);
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 
 	gtk_text_buffer_insert_with_tags_by_name (priv->buffer, &iter,
 						  msg, -1,
@@ -1499,18 +1454,187 @@ gossip_chat_view_append_event (GossipChatView *view,
 	}
 
 	priv->last_block_type = BLOCK_TYPE_EVENT;
-	priv->last_timestamp = timestamp;
 }
 
 void
-gossip_chat_view_clear (GossipChatView *view)
+gossip_chat_view_append_invite (GossipChatView *view,
+				GossipMessage  *message)
 {
-	GtkTextBuffer *buffer;
+	GossipChatViewPriv   *priv;
+	GossipContact        *sender;
+	GossipChatroomInvite *invite;
+	const gchar          *body;
+	GtkTextChildAnchor   *anchor;
+	GtkTextIter           iter;
+	GtkWidget            *button_accept;
+	GtkWidget            *button_decline;
+	const gchar          *id;
+	const gchar          *reason;
+	gboolean              bottom;
+	const gchar          *tag;
+	GString              *s;
 
-	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	priv = GET_PRIV (view);
+
+	if (priv->irc_style) {
+		tag = "irc-invite";
+	} else {
+		tag = "fancy-invite";
+	}
+	
+	bottom = chat_view_is_scrolled_down (view);
+
+	sender = gossip_message_get_sender (message);
+	invite = gossip_message_get_invite (message);
+	body = gossip_message_get_body (message);
+
+	chat_view_maybe_append_date_and_time (view, message);
+
+	reason = gossip_chatroom_invite_get_reason (invite);
+
+	s = g_string_new ("");
+	if (body && strlen (body) > 0) {
+		s = g_string_append (s, body);
+	}
+
+	/* Make sure the reason is not the body (or in the body) */
+	if (reason && strlen (reason) > 0 && !strstr (body, reason)) {
+		if (s->len > 0) {
+			s = g_string_append_c (s, '\n');
+		}
+
+		s = g_string_append (s, reason);
+	}
+	
+	if (s->len < 1) {
+		s = g_string_append 
+			(s, _("You have been invited to join a chat conference."));
+	}
+
+	/* Don't include the invite in the chat window if it is part of the
+	 * actual request - some chat clients send this and it looks weird
+	 * repeated.
+	 */
+	id = gossip_chatroom_invite_get_id (invite);
+
+	if (!strstr (s->str, id)) {
+		g_string_append_printf (s, "\n(%s)\n", id);
+	}
+
+	s = g_string_prepend_c (s, '\n');
+	
+	chat_view_append_text (view, s->str, tag);
+	g_string_free (s, TRUE);
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+
+	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+
+	button_accept = gtk_button_new_with_label (_("Accept"));
+	gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view),
+					   button_accept,
+					   anchor);
+
+	gtk_widget_show (button_accept);
+
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  " ",
+						  1,
+						  tag,
+						  NULL);
+
+	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+
+	button_decline = gtk_button_new_with_label (_("Decline"));
+	gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view),
+					   button_decline,
+					   anchor);
+
+	gtk_widget_show (button_decline);
+
+	/* Set up data */
+	g_object_set_data_full (G_OBJECT (button_accept), "invite",
+				gossip_chatroom_invite_ref (invite), 
+				(GDestroyNotify) gossip_chatroom_invite_unref);
+ 	g_object_set_data_full (G_OBJECT (button_accept), "other_button",
+ 				g_object_ref (button_decline), 
+				g_object_unref);
+
+	g_signal_connect (button_accept, "clicked",
+			  G_CALLBACK (chat_view_invite_accept_cb),
+			  NULL);
+
+	g_object_set_data_full (G_OBJECT (button_decline), "invite",
+				gossip_chatroom_invite_ref (invite), 
+				(GDestroyNotify) gossip_chatroom_invite_unref);
+ 	g_object_set_data_full (G_OBJECT (button_decline), "other_button",
+ 				g_object_ref (button_accept), 
+				g_object_unref);
+
+	g_signal_connect (button_decline, "clicked",
+			  G_CALLBACK (chat_view_invite_decline_cb),
+			  NULL);
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  "\n\n",
+						  2,
+						  tag,
+						  NULL);
+
+	if (bottom) {
+		gossip_chat_view_scroll_down (view);
+	}
+
+	priv->last_block_type = BLOCK_TYPE_INVITE;
+}
+
+void
+gossip_chat_view_scroll (GossipChatView *view,
+			 gboolean        allow_scrolling)
+{
+	GossipChatViewPriv *priv;
+
+	priv = GET_PRIV (view);
+
+	priv->allow_scrolling = allow_scrolling;
+
+	DEBUG_MSG (("ChatView: Scrolling %s", allow_scrolling ? "enabled" : "disabled"));
+}
+
+void
+gossip_chat_view_scroll_down (GossipChatView *view)
+{
+	GossipChatViewPriv *priv;
+
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter;
+	GtkTextMark        *mark;
+
+	priv = GET_PRIV (view);
+
+	if (!priv->allow_scrolling) {
+		return;
+	}
+
+	DEBUG_MSG (("ChatView: Scrolling down"));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	gtk_text_buffer_set_text (buffer, "", -1);
+
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	mark = gtk_text_buffer_create_mark (buffer,
+					    NULL,
+					    &iter,
+					    FALSE);
+
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+				      mark,
+				      0.0,
+				      FALSE,
+				      0,
+				      0);
 }
 
 gboolean
@@ -1527,6 +1651,157 @@ gossip_chat_view_get_selection_bounds (GossipChatView *view,
 	return gtk_text_buffer_get_selection_bounds (buffer, start, end);
 }
 
+void
+gossip_chat_view_clear (GossipChatView *view)
+{
+	GtkTextBuffer      *buffer;
+	GossipChatViewPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	gtk_text_buffer_set_text (buffer, "", -1);
+
+	/* We set these back to the initial values so we get
+	 * timestamps when clearing the window to know when
+	 * conversations start.
+	 */
+	priv = GET_PRIV (view);
+	
+	priv->last_block_type = BLOCK_TYPE_NONE;
+	priv->last_timestamp = 0;
+}
+
+void
+gossip_chat_view_find (GossipChatView *view, 
+		       const gchar    *search_criteria,
+		       gboolean        new_search)
+{
+	GtkTextBuffer      *buffer;
+	GtkTextIter         iter_at_mark;
+	GtkTextIter         iter_match_start;
+	GtkTextIter         iter_match_end;
+
+	static GtkTextMark *find_mark = NULL;
+	static gboolean     wrapped = FALSE;
+        
+	gboolean            found;
+	gboolean            from_start = FALSE;
+
+	gchar              *str;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+	g_return_if_fail (search_criteria != NULL);
+	g_return_if_fail (strlen (search_criteria) > 0);
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+    
+	if (new_search) {
+		find_mark = NULL;
+		from_start = TRUE;
+	}
+     
+	if (find_mark) {
+		gtk_text_buffer_get_iter_at_mark (buffer, &iter_at_mark, find_mark);
+		gtk_text_buffer_delete_mark (buffer, find_mark); 
+		find_mark = NULL;
+	} else {
+		gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
+		from_start = TRUE;
+	}
+
+	str = g_utf8_casefold (search_criteria, -1);
+	found = gtk_text_iter_forward_search (&iter_at_mark, 
+					      str, 
+					      GTK_TEXT_SEARCH_TEXT_ONLY, 
+					      &iter_match_start, 
+					      &iter_match_end,
+					      NULL);
+	g_free (str);
+    
+	if (!found) {
+		if (from_start) {
+			return;
+		}
+	
+		/* Here we wrap around. */
+		if (!new_search && !wrapped) {
+			wrapped = TRUE;
+			gossip_chat_view_find (view, search_criteria, FALSE);
+			wrapped = FALSE;
+		}
+
+		return;
+	}
+    
+	/* Set new mark and show on screen */
+	find_mark = gtk_text_buffer_create_mark (buffer, NULL, &iter_match_end, TRUE); 
+	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
+				      find_mark,
+				      0.0,
+				      TRUE,
+				      0.5,
+				      0.5);
+
+	gtk_text_buffer_move_mark_by_name (buffer, "selection_bound", &iter_match_start);
+	gtk_text_buffer_move_mark_by_name (buffer, "insert", &iter_match_end);	
+}
+
+void
+gossip_chat_view_highlight (GossipChatView *view, 
+			    const gchar    *text)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter    iter;
+	GtkTextIter    iter_start;
+	GtkTextIter    iter_end;
+	GtkTextIter    iter_match_start;
+	GtkTextIter    iter_match_end;
+	        
+	gboolean       found;
+
+	gchar         *str;
+
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+    
+	gtk_text_buffer_get_start_iter (buffer, &iter);
+
+	gtk_text_buffer_get_bounds (buffer, &iter_start, &iter_end);
+	gtk_text_buffer_remove_tag_by_name (buffer, "highlight", 
+					    &iter_start, 
+					    &iter_end);
+
+	if (!text || strlen (text) < 1) {
+		return;
+	}
+
+	str = g_utf8_casefold (text, -1);
+
+	while (TRUE) {
+		found = gtk_text_iter_forward_search (&iter, 
+						      str, 
+						      GTK_TEXT_SEARCH_TEXT_ONLY, 
+						      &iter_match_start, 
+						      &iter_match_end,
+						      NULL);
+
+		if (!found) {
+			break;
+		}
+
+		gtk_text_buffer_apply_tag_by_name (buffer, "highlight", 
+						   &iter_match_start, 
+						   &iter_match_end);
+
+		iter = iter_match_end;
+		gtk_text_iter_forward_char (&iter);
+	}
+
+	g_free (str);
+}
+
 void
 gossip_chat_view_copy_clipboard (GossipChatView *view)
 {
@@ -1541,53 +1816,16 @@ gossip_chat_view_copy_clipboard (GossipChatView *view)
 	gtk_text_buffer_copy_clipboard (buffer, clipboard);
 }
 
-static void
-chat_view_invite_accept_cb (GtkWidget *button,
-			    gpointer   user_data)
-{
-	GossipSession          *session;
-	GossipAccount          *account;
-	GossipContact          *contact;
-	GossipChatroomProvider *provider;
-	const gchar            *nickname;
-	const gchar            *invite;
-
-	invite = g_object_get_data (G_OBJECT (button), "invite");
-	contact = g_object_get_data (G_OBJECT (button), "contact");
-
-	gtk_widget_set_sensitive (button, FALSE);
-
-	session = gossip_app_get_session ();
-	account = gossip_contact_get_account (contact);
-	provider = gossip_session_get_chatroom_provider (session, account);
-	nickname = gossip_session_get_nickname (session, account);
-
-	gossip_chatroom_provider_invite_accept (provider,
-						chat_view_invite_join_cb,
-						nickname,
-						invite);
-}
-
-static void
-chat_view_invite_join_cb (GossipChatroomProvider   *provider,
-			  GossipChatroomJoinResult  result,
-			  gint                      id,
-			  gpointer                  user_data)
-{
-	gossip_group_chat_show (provider, id);
-}
-
-static void
-theme_manager_theme_changed_cb (GossipThemeManager *manager,
-				GossipChatView     *view)
+gboolean
+gossip_chat_view_get_irc_style (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
 	
-	priv = view->priv;
-	
-	priv->last_block_type = BLOCK_TYPE_NONE;
-	
-	gossip_theme_manager_apply (manager, view);
+	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+
+	priv = GET_PRIV (view);
+
+	return priv->irc_style;
 }
 
 void
@@ -1599,7 +1837,7 @@ gossip_chat_view_set_irc_style (GossipChatView *view,
 	
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
-	priv = view->priv;
+	priv = GET_PRIV (view);
 
 	priv->irc_style = irc_style;
 
@@ -1615,14 +1853,93 @@ gossip_chat_view_set_irc_style (GossipChatView *view,
 		      NULL);
 }
 
-gboolean
-gossip_chat_view_get_irc_style (GossipChatView *view)
+GdkPixbuf *
+gossip_chat_view_get_smiley_image (GossipSmiley smiley)
 {
-	GossipChatViewPriv *priv;
-	
-	g_return_val_if_fail (GOSSIP_IS_CHAT_VIEW (view), FALSE);
+	static GdkPixbuf *pixbufs[GOSSIP_SMILEY_COUNT];
+	static gboolean   inited = FALSE;
 
-	priv = view->priv;
+	if (!inited) {
+		gint i;
 
-	return priv->irc_style;
+		for (i = 0; i < GOSSIP_SMILEY_COUNT; i++) {
+			pixbufs[i] = gossip_pixbuf_from_smiley (i, GTK_ICON_SIZE_MENU);
+		}
+
+		inited = TRUE;
+	}
+
+	return pixbufs[smiley];
+}
+
+const gchar *
+gossip_chat_view_get_smiley_text (GossipSmiley smiley)
+{
+	gint i;
+	
+	for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
+		if (smileys[i].smiley != smiley) {
+			continue;
+		}
+
+		return smileys[i].pattern;
+	}
+	
+	return NULL;
+}
+
+GtkWidget *
+gossip_chat_view_get_smiley_menu (GCallback callback, 
+				  gpointer  user_data)
+{
+	GtkWidget *menu;
+	gint       x;
+	gint       y;
+	gint       i;
+
+	/* Set up smiley menu */
+	menu = gtk_menu_new ();
+
+	for (i = 0, x = 0, y = 0; i < GOSSIP_SMILEY_COUNT; i++) {
+		GtkWidget   *item;
+		GtkWidget   *image;
+		GdkPixbuf   *pixbuf;
+		GtkTooltips *tooltips;
+		const gchar *smiley_text;
+
+		pixbuf = gossip_chat_view_get_smiley_image (i);
+		if (!pixbuf) {
+			continue;
+		}
+
+		image = gtk_image_new_from_pixbuf (pixbuf);
+
+		item = gtk_image_menu_item_new_with_label ("");
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+		gtk_menu_attach (GTK_MENU (menu), item,
+				 x, x + 1, y, y + 1);
+
+		smiley_text = gossip_chat_view_get_smiley_text (i);
+
+		tooltips = gtk_tooltips_new ();
+		gtk_tooltips_set_tip (tooltips, 
+				      item, 
+				      smiley_text, 
+				      NULL);
+
+		g_object_set_data  (G_OBJECT (item), "smiley_text", (gpointer) smiley_text);
+		g_signal_connect (item, "activate", callback, user_data);
+		
+		if (x > 3) {
+			y++;
+			x = 0;
+		} else {
+			x++;
+		}
+	}
+
+	gtk_widget_show_all (menu);
+
+	return menu;
 }
