@@ -36,10 +36,11 @@
 #include "gossip-chat.h"
 #include "gossip-app.h"
 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT, GossipChatPriv))
+
 #define DEBUG_MSG(x)  
 /* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
 
-
 struct _GossipChatPriv {
 	GossipChatWindow *window;
 
@@ -52,7 +53,6 @@ struct _GossipChatPriv {
 	gint              last_input_height;
 };
 
-
 typedef struct {
 	GossipChat  *chat;
        	gchar       *word;
@@ -61,7 +61,6 @@ typedef struct {
        	GtkTextIter  end;
 } GossipChatSpell;
 
-
 static void      gossip_chat_class_init            (GossipChatClass *klass);
 static void      gossip_chat_init                  (GossipChat      *chat);
 static void      chat_finalize                     (GObject         *object);
@@ -81,7 +80,6 @@ GossipChatSpell *chat_spell_new                    (GossipChat      *chat,
 						    GtkTextIter      end);
 static void      chat_spell_free                   (GossipChatSpell *chat_spell);
 
-
 enum {
 	COMPOSING,
 	NEW_MESSAGE,
@@ -90,19 +88,18 @@ enum {
 	LAST_SIGNAL
 };
 
+static guint chat_signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
-static guint         chat_signals[LAST_SIGNAL] = { 0 };
-
 
 G_DEFINE_TYPE (GossipChat, gossip_chat, G_TYPE_OBJECT);
 
-
 static void
 gossip_chat_class_init (GossipChatClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass *object_class;
 
+	object_class = G_OBJECT_CLASS (klass);
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = chat_finalize;
@@ -115,8 +112,7 @@ gossip_chat_class_init (GossipChatClass *klass)
                               NULL, NULL,
 			      gossip_marshal_VOID__BOOLEAN,
                               G_TYPE_NONE,
-                              1,
-                              G_TYPE_BOOLEAN);
+                              1, G_TYPE_BOOLEAN);
 
 	chat_signals[NEW_MESSAGE] =
 		g_signal_new ("new-message",
@@ -124,9 +120,9 @@ gossip_chat_class_init (GossipChatClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GossipChatClass, new_message),
 			      NULL, NULL,
-			      gossip_marshal_VOID__VOID,
+			      gossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      0);
+			      1, GOSSIP_TYPE_MESSAGE);
 
 	chat_signals[NAME_CHANGED] =
 		g_signal_new ("name-changed",
@@ -147,6 +143,8 @@ gossip_chat_class_init (GossipChatClass *klass)
 			      gossip_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+
+	g_type_class_add_private (object_class, sizeof (GossipChatPriv));
 }
 
 static void
@@ -171,10 +169,9 @@ gossip_chat_init (GossipChat *chat)
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
 		      NULL);
 
-	priv = g_new0 (GossipChatPriv, 1);
-	priv->default_window_height = -1;
+	priv = GET_PRIV (chat);
 
-	chat->priv = priv;
+	priv->default_window_height = -1;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	g_signal_connect (buffer,
@@ -236,7 +233,7 @@ chat_finalize (GObject *object)
 	g_return_if_fail (GOSSIP_IS_CHAT (object));
 
 	chat = GOSSIP_CHAT (object);
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
 	if (priv->spell) {
 		gossip_spell_unref (priv->spell);
@@ -255,7 +252,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 	gchar          *str;
 	gboolean        spell_checker;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
  	spell_checker = gconf_client_get_bool (gossip_app_get_gconf_client (), 
 					       "/apps/gossip/conversation/enable_spell_checker", 
@@ -371,7 +368,7 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 	gint              current_height;
 	gint              diff;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
 	if (priv->default_window_height <= 0) {
 		return;
@@ -415,24 +412,62 @@ chat_text_view_size_allocate_cb (GtkWidget     *widget,
 			 data, g_free);
 }
 
+static void
+chat_insert_smiley_activate_cb (GtkWidget  *menuitem, 
+				GossipChat *chat)
+{
+	GtkTextBuffer *buffer;
+	GtkTextIter    iter;
+	const gchar   *smiley;
+
+	smiley = g_object_get_data (G_OBJECT (menuitem), "smiley_text");
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
+	gtk_text_buffer_get_end_iter (buffer, &iter);
+	gtk_text_buffer_insert (buffer, &iter, 
+				smiley, 
+				g_utf8_strlen (smiley, -1));
+}
+
 static void
 chat_text_populate_popup_cb (GtkTextView *view,
 			     GtkMenu     *menu,
 			     GossipChat  *chat)
 {
-	GtkTextBuffer      *buffer;
-	GtkTextTagTable    *table;
-	GtkTextTag         *tag;
-	gint                x, y;
-	GtkTextIter         iter, start, end;
-	GtkWidget          *item;
-	gchar              *str = NULL;
-	GossipChatSpell    *chat_spell;
+	GtkTextBuffer   *buffer;
+	GtkTextTagTable *table;
+	GtkTextTag      *tag;
+	gint             x, y;
+	GtkTextIter      iter, start, end;
+	GtkWidget       *item;
+	gchar           *str = NULL;
+	GossipChatSpell *chat_spell;
+	GtkWidget       *smiley_menu;
+
+	/*
+	 * Add the emoticon menu.
+	 */
+
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+	
+	item = gtk_menu_item_new_with_mnemonic (_("Insert Smiley"));
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	smiley_menu = gossip_chat_view_get_smiley_menu (
+		G_CALLBACK (chat_insert_smiley_activate_cb), 
+		chat);
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
+
+	/*
+	 * Add the spell check menu item.
+	 */
 
 	buffer = gtk_text_view_get_buffer (view);
 	table = gtk_text_buffer_get_tag_table (buffer);
-	
-	/* handle misspelled tags */
+
 	tag = gtk_text_tag_table_lookup (table, "misspelled");
 	
 	gtk_widget_get_pointer (GTK_WIDGET (view), &x, &y);
@@ -514,7 +549,7 @@ chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem,
 {
 	GossipChatPriv *priv;
 
-	priv = chat_spell->chat->priv;
+	priv = GET_PRIV (chat_spell->chat);
 
 	gossip_spell_dialog_show (chat_spell->chat,
 				  priv->spell,
@@ -690,15 +725,23 @@ gossip_chat_clear (GossipChat *chat)
 }
 
 void
-gossip_chat_set_window (GossipChat *chat, GossipChatWindow *window)
+gossip_chat_set_window (GossipChat       *chat, 
+			GossipChatWindow *window)
 {
-	chat->priv->window = window;
+	GossipChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+	priv->window = window;
 }
 
 GossipChatWindow *
 gossip_chat_get_window (GossipChat *chat)
 {
-	return chat->priv->window;
+	GossipChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+
+	return priv->window;
 }
 
 void
@@ -712,7 +755,7 @@ gossip_chat_scroll_down (GossipChat *chat)
 void
 gossip_chat_cut (GossipChat *chat)
 {
-	GtkTextBuffer  *buffer;
+	GtkTextBuffer *buffer;
 	
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -729,7 +772,7 @@ gossip_chat_cut (GossipChat *chat)
 void
 gossip_chat_copy (GossipChat *chat)
 {
-	GtkTextBuffer         *buffer;
+	GtkTextBuffer *buffer;
 	
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -751,8 +794,8 @@ gossip_chat_copy (GossipChat *chat)
 void
 gossip_chat_paste (GossipChat *chat)
 {
-	GtkTextBuffer         *buffer;
-	GtkClipboard          *clipboard;
+	GtkTextBuffer *buffer;
+	GtkClipboard  *clipboard;
 
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
@@ -769,7 +812,7 @@ gossip_chat_present (GossipChat *chat)
 
 	g_return_if_fail (GOSSIP_IS_CHAT (chat));
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	if (priv->window == NULL) {
 		GossipChatWindow *window;
