@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <string.h>
-
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
+#include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
@@ -38,6 +34,7 @@
 #include "gossip-edit-groups-dialog.h"
 #include "gossip-ft-window.h"
 #include "gossip-log.h"
+#include "gossip-log-window.h"
 #include "gossip-marshal.h"
 #include "gossip-sound.h"
 #include "gossip-stock.h"
@@ -46,34 +43,28 @@
 #include "gossip-notify.h"
 #endif
 
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT_LIST, GossipContactListPriv))
+
 #define DEBUG_MSG(x)  
 /* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
 
 /* Flashing delay for icons (milliseconds). */
 #define FLASH_TIMEOUT 500
 
-/* 
- * Active users are those which have recently changed state
+/* Active users are those which have recently changed state
  * (e.g. online, offline or from normal to a busy state). 
- * 
  */
 
-/* time user is shown as active */
+/* Time user is shown as active */
 #define ACTIVE_USER_SHOW_TIME 7000     
         
-/* time after connecting which we wait before active users are enabled */
+/* Time after connecting which we wait before active users are enabled */
 #define ACTIVE_USER_WAIT_TO_ENABLE_TIME 5000    
 
-/* time to wait before we use sounds for an account after it has gone
-   online/offline, so we don't spam the sound with online's, etc */
-#define SOUNDS_ENABLED_WAIT_TIME 10000   
-
-
 struct _GossipContactListPriv {
 	gboolean             show_offline;
 	gboolean             show_active;
 
-	/* protocol to group association */
 	GHashTable          *groups;
 
         GtkItemFactory      *item_popup_factory;
@@ -84,41 +75,35 @@ struct _GossipContactListPriv {
 	GtkTreeRowReference *drag_row;
 };
 
-
 typedef struct {
 	gboolean flash_on;
 	guint    flash_timeout_id;
 } FlashData;
 
-
 typedef struct {
 	gchar       *name;
 	gboolean     found;
 	GtkTreeIter  iter;
 } FindGroup;
 
-
 typedef struct {
 	GossipContact *contact;
 	gboolean       found;
 	GList         *iters;
 } FindContact;
 
-
 typedef struct {
 	GossipContactList *list;
 	GtkTreePath       *path;
 	guint              timeout_id;
 } DragMotionData;
 
-
 typedef struct {
 	GossipContactList *list;
 	GossipContact     *contact;
 	gboolean           remove;
 } ShowActiveData;
 
-
 static void     gossip_contact_list_class_init           (GossipContactListClass *klass);
 static void     gossip_contact_list_init                 (GossipContactList *list);
 static void     contact_list_finalize                    (GObject           *object);
@@ -269,8 +254,6 @@ static void     contact_list_event_removed_cb            (GossipEventManager *ma
 							  GossipContactList  *list);
 static void     contact_list_flash_free_data             (FlashData          *data);
 
-
-/* signals */
 enum {
         CONTACT_ACTIVATED,
         LAST_SIGNAL
@@ -278,8 +261,6 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-
-/* Model columns */
 enum {
 	COL_PIXBUF,
 	COL_NAME,
@@ -292,15 +273,11 @@ enum {
 	COL_COUNT
 };
 
-
-/* Properties */
 enum {
 	PROP_0,
 	PROP_SHOW_OFFLINE
 };
 
-
-/* Item context menu */
 enum {
         ITEM_MENU_NONE,
         ITEM_MENU_INFO,
@@ -312,18 +289,15 @@ enum {
         ITEM_MENU_LOG
 };
 
-
-/* Group context menu */
 enum {
         GROUP_MENU_NONE,
         GROUP_MENU_RENAME
 };
 
-
 #define GIF_CB(x)    ((GtkItemFactoryCallback)(x))
 
 static GtkItemFactoryEntry item_menu_items[] = {
-	{ N_("/Contact _Information"),
+	{ N_("/Contact Infor_mation"),
 	  NULL,
 	  GIF_CB (contact_list_item_menu_info_cb),
 	  ITEM_MENU_INFO,
@@ -359,7 +333,7 @@ static GtkItemFactoryEntry item_menu_items[] = {
 	  0,
 	  "<Separator>",
 	  NULL },
-	{ N_("/_Invite to Chat Conference"),
+	{ N_("/_Invite to Chat Room"),
 	  NULL,
 	  NULL,
 	  ITEM_MENU_INVITE,
@@ -383,15 +357,14 @@ static GtkItemFactoryEntry item_menu_items[] = {
 	  0,
 	  "<Separator>",
 	  NULL },
-	{ N_("/Show _Log"),
+	{ N_("/_View Previous Conversations"),
 	  NULL,
 	  GIF_CB (contact_list_item_menu_log_cb),
 	  ITEM_MENU_LOG,
-	  "<Item>",
-	  NULL },
+	  "<StockItem>",
+	  GTK_STOCK_JUSTIFY_LEFT }
 };
 
-
 static GtkItemFactoryEntry group_menu_items[] = {
 	{ N_("/Re_name group"),
 	  NULL,
@@ -401,14 +374,12 @@ static GtkItemFactoryEntry group_menu_items[] = {
 	  NULL }
 };
 
-
 enum DndDragType {
 	DND_DRAG_TYPE_STRING,
 	DND_DRAG_TYPE_URL,
 	DND_DRAG_TYPE_CONTACT_ID,
 };
 
-
 static GtkTargetEntry drag_types_dest[] = {
 	{ "STRING",          0, DND_DRAG_TYPE_STRING },
 	{ "text/plain",      0, DND_DRAG_TYPE_STRING },
@@ -416,19 +387,15 @@ static GtkTargetEntry drag_types_dest[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
-
 static GtkTargetEntry drag_types_source[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
-
 static GdkAtom drag_atoms_dest[G_N_ELEMENTS (drag_types_dest)];
 static GdkAtom drag_atoms_source[G_N_ELEMENTS (drag_types_source)];
 
-
 G_DEFINE_TYPE (GossipContactList, gossip_contact_list, GTK_TYPE_TREE_VIEW);
 
-
 static void
 gossip_contact_list_class_init (GossipContactListClass *klass)
 {
@@ -459,6 +426,8 @@ gossip_contact_list_class_init (GossipContactListClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_OFFLINE,
 					 param);
+
+	g_type_class_add_private (object_class, sizeof (GossipContactListPriv));
 }
 
 static void
@@ -466,8 +435,7 @@ gossip_contact_list_init (GossipContactList *list)
 {
 	GossipContactListPriv *priv;
 	
-	priv = g_new0 (GossipContactListPriv, 1);
-	list->priv = priv;
+	priv = GET_PRIV (list);
 
 	priv->groups = g_hash_table_new_full (g_str_hash, 
 					      g_str_equal,
@@ -482,10 +450,10 @@ gossip_contact_list_init (GossipContactList *list)
 	contact_list_create_model (list);
 	contact_list_setup_view (list);
 
-	/* get saved group states */
+	/* Get saved group states. */
 	gossip_contact_groups_get_all ();
 
-        /* context menus */
+        /* Context menus. */
         priv->item_popup_factory = gtk_item_factory_new (GTK_TYPE_MENU,
 							 "<main>", NULL);
 	priv->group_popup_factory = gtk_item_factory_new (GTK_TYPE_MENU,
@@ -511,7 +479,7 @@ gossip_contact_list_init (GossipContactList *list)
 				       group_menu_items,
 				       list);
         
-        /* signal connection */
+        /* Signal connection. */
 	g_signal_connect (gossip_app_get_session (),
 			  "connected",
 			  G_CALLBACK (contact_list_connected_cb),
@@ -533,11 +501,11 @@ gossip_contact_list_init (GossipContactList *list)
 			  G_CALLBACK (contact_list_contact_removed_cb),
 			  list);
 	g_signal_connect (gossip_app_get_session (),
-			  "composing-event",
+			  "composing",
 			  G_CALLBACK (contact_list_contact_composing_cb),
 			  list);
 
-	/* connect to event manager signals */
+	/* Connect to event manager signals. */
 	g_signal_connect (gossip_app_get_event_manager (),
 			  "event-added",
 			  G_CALLBACK (contact_list_event_added_cb),
@@ -547,7 +515,7 @@ gossip_contact_list_init (GossipContactList *list)
 			  G_CALLBACK (contact_list_event_removed_cb),
 			  list);
 
-	/* connect to tree view signals rather than override */
+	/* Connect to tree view signals rather than override. */
 	g_signal_connect (list,
 			  "button-press-event",
 			  G_CALLBACK (contact_list_button_press_event_cb),
@@ -579,7 +547,7 @@ contact_list_get_property (GObject    *object,
 {
 	GossipContactListPriv *priv;
 
-	priv = GOSSIP_CONTACT_LIST (object)->priv;
+	priv = GET_PRIV (object);
 
 	switch (param_id) {
 	case PROP_SHOW_OFFLINE:
@@ -599,7 +567,7 @@ contact_list_set_property (GObject      *object,
 {
 	GossipContactListPriv *priv;
 
-	priv = GOSSIP_CONTACT_LIST (object)->priv;
+	priv = GET_PRIV (object);
 
 	switch (param_id) {
 	case PROP_SHOW_OFFLINE:
@@ -625,7 +593,9 @@ contact_list_connected_cb (GossipSession *session, GossipContactList *list)
 static gboolean
 contact_list_show_active_users_cb (GossipContactList *list)
 {
-	GossipContactListPriv *priv = list->priv;
+	GossipContactListPriv *priv;
+
+	priv = GET_PRIV (list);
 
 	priv->show_active = TRUE;
 
@@ -641,7 +611,7 @@ contact_list_contact_added_cb (GossipSession     *session,
 	GtkTreeModel          *model;
 	GList                 *groups;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	DEBUG_MSG (("ContactList: Contact added: %s",
 		   gossip_contact_get_name (contact)));
@@ -667,7 +637,7 @@ contact_list_contact_updated_cb (GossipSession     *session,
 	GossipPresence        *presence;
 	GtkTreeModel          *model;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	type = gossip_contact_get_type (contact);
 	if (type != GOSSIP_CONTACT_TYPE_CONTACTLIST) {
@@ -686,9 +656,10 @@ contact_list_contact_updated_cb (GossipSession     *session,
 		return;
 	}
 
-	/* we do this to make sure the groups are correct, if not, we
-	   would have to check the groups already set up for each
-	   contact and then see what has been updated */
+	/* We do this to make sure the groups are correct, if not, we
+	 * would have to check the groups already set up for each
+	 * contact and then see what has been updated. 
+	 */
 	contact_list_remove_contact (list, contact);
 	contact_list_add_contact (list, contact);
 }
@@ -713,14 +684,14 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 	gboolean               do_set_refresh = FALSE;
 
 	GossipAccount         *account;
-	gdouble                seconds;
+	gdouble                seconds = 0;
 
 	gboolean               is_composing;
 	const GdkPixbuf       *pixbuf;
  	GdkPixbuf             *pixbuf_composing;
 	GdkPixbuf             *pixbuf_presence;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
 	type = gossip_contact_get_type (contact);
@@ -743,21 +714,20 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		should_be_in_list = FALSE;
 	}
 
-	/* get online state now */
+	/* Get online state now. */
 	now_online = gossip_contact_is_online (contact);
 
 	if (!in_list && !should_be_in_list) {
-		/* nothing to do */
+		/* Nothing to do. */
 		g_list_foreach (iters, (GFunc)gtk_tree_iter_free, NULL);
 		g_list_free (iters);
 		return;
 	}
 	else if (in_list && !should_be_in_list) {
 		account = gossip_session_find_account (session, contact);
-		seconds = gossip_session_get_connected_time (session, account);
-
-		if (seconds >= (SOUNDS_ENABLED_WAIT_TIME / 1000)) {
-			gossip_sound_play (GOSSIP_SOUND_OFFLINE);
+		if (account) {
+			seconds = gossip_session_get_connected_time (session, 
+								     account);
 		}
 
 		if (priv->show_active) {
@@ -774,14 +744,9 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 	}
 	else if (!in_list && should_be_in_list) {
 		account = gossip_session_find_account (session, contact);
-		seconds = gossip_session_get_connected_time (session, account);
-
-		if (seconds >= (SOUNDS_ENABLED_WAIT_TIME / 1000)) {
-			gossip_sound_play (GOSSIP_SOUND_ONLINE);
-
-#ifdef HAVE_LIBNOTIFY
-			gossip_notify_contact_online (contact);
-#endif
+		if (account) {
+			seconds = gossip_session_get_connected_time (session, 
+								     account);
 		}
 
 		contact_list_add_contact (list, contact);
@@ -793,7 +758,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		}
 
 	} else {
-		/* get online state before */
+		/* Get online state before. */
 		if (iters && g_list_length (iters) > 0) {
 			GtkTreeIter *iter;
 			
@@ -801,7 +766,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 			gtk_tree_model_get (model, iter, COL_IS_ONLINE, &was_online, -1);
 		}
 
-		/* is this really an update or an online/offline */
+		/* Is this really an update or an online/offline. */
 		if (priv->show_active) {
 			if (was_online != now_online) {
 				gchar *str;
@@ -818,7 +783,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 				DEBUG_MSG (("ContactList: Set active (contact updated %s)", str)); 
 
 			} else {
-				/* was TRUE for presence updates */
+				/* Was TRUE for presence updates. */
 				/* do_set_active = FALSE;  */
 				do_set_refresh = TRUE;
 				
@@ -829,7 +794,6 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		set_model = TRUE;
 	}
 
-
 	pixbuf_presence = gossip_pixbuf_for_contact (contact);
 	pixbuf_composing = gossip_pixbuf_from_stock (GOSSIP_STOCK_TYPING, 
 						     GTK_ICON_SIZE_MENU);
@@ -867,10 +831,10 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 	}
 
 	/* FIXME: when someone goes online then offline quickly, the
-	   first timeout sets the user to be inactive and the
-	   second timeout removes the user from the contact
-	   list, really we should remove the first timeout. */
-		
+	 * first timeout sets the user to be inactive and the second
+	 * timeout removes the user from the contact list, really we
+	 * should remove the first timeout. 
+	 */		
 	g_list_foreach (iters, (GFunc)gtk_tree_iter_free, NULL);
 	g_list_free (iters);
 }
@@ -901,7 +865,7 @@ contact_list_contact_composing_cb (GossipSession     *session,
 		   composing ? "is" : "is not",
 		   gossip_contact_get_name (contact)));
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
@@ -1016,7 +980,7 @@ contact_list_contact_active_cb (ShowActiveData *data)
 	
 	g_return_val_if_fail (data != NULL, FALSE);
 
-	priv = data->list->priv;
+	priv = GET_PRIV (data->list);
 
 	if (data->remove && 
 	    !priv->show_offline && 
@@ -1135,7 +1099,7 @@ contact_list_get_group_foreach (GtkTreeModel *model,
 	gchar    *str;
 	gboolean  is_group;
 
-	/* groups are only at the top level */
+	/* Groups are only at the top level. */
 	if (gtk_tree_path_get_depth (path) != 1) {
 		return FALSE;
 	}
@@ -1163,11 +1127,11 @@ contact_list_add_contact (GossipContactList *list,
 	GtkTreeModel          *model;
 	GList                 *l, *groups;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
-	/* if no groups just add it at the top level */
+	/* If no groups just add it at the top level. */
 	groups = gossip_contact_get_groups (contact);
 	if (!groups) {
 		GdkPixbuf *pixbuf;
@@ -1189,7 +1153,7 @@ contact_list_add_contact (GossipContactList *list,
 		g_object_unref (pixbuf);
 	}
 
-	/* else add to each group */
+	/* Else add to each group. */
 	for (l = groups; l; l = l->next) {
 		GtkTreePath *path;
 		GdkPixbuf   *pixbuf;
@@ -1260,7 +1224,7 @@ contact_list_remove_contact (GossipContactList *list,
 	GtkTreeModel          *model;
 	GList                 *iters, *l;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 	
 	iters = contact_list_find_contact (list, contact);
 	if (!iters) {
@@ -1325,7 +1289,6 @@ contact_list_setup_view (GossipContactList *list)
 		      "reorderable", TRUE,
 		      NULL);
 
-	/* columns, cells, etc */
 	col  = gtk_tree_view_column_new ();
 	
 	cell = gtk_cell_renderer_pixbuf_new ();
@@ -1359,7 +1322,7 @@ contact_list_setup_view (GossipContactList *list)
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (list), col);
 
-	/* drag & drop */ 
+	/* Drag & Drop. */ 
 	for (i = 0; i < G_N_ELEMENTS (drag_types_dest); ++i) {
 		drag_atoms_dest[i] = gdk_atom_intern (drag_types_dest[i].target,
 						      FALSE);
@@ -1370,17 +1333,20 @@ contact_list_setup_view (GossipContactList *list)
 							FALSE);
 	}
 
+	/* Note: We support the COPY action too, but need to make the
+	 * MOVE action the default.
+	 */
 	gtk_drag_source_set (GTK_WIDGET (list), 
 			     GDK_BUTTON1_MASK, 
 			     drag_types_source, 
 			     G_N_ELEMENTS (drag_types_source), 
-			     GDK_ACTION_MOVE | GDK_ACTION_COPY);
+			     GDK_ACTION_MOVE);
 
 	gtk_drag_dest_set (GTK_WIDGET (list), 
 			   GTK_DEST_DEFAULT_ALL, 
 			   drag_types_dest, 
 			   G_N_ELEMENTS (drag_types_dest),
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+			   GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	
 	g_signal_connect (GTK_WIDGET (list),
 			  "drag-data-received",
@@ -1388,8 +1354,9 @@ contact_list_setup_view (GossipContactList *list)
 			  NULL);
 
 	/* FIXME: noticed but when you drag the row over the treeview
-	   fast, it seems to stop redrawing itself, if we don't
-	   connect this signal, all is fine. */
+	 * fast, it seems to stop redrawing itself, if we don't
+	 * connect this signal, all is fine. 
+	 */
  	g_signal_connect (GTK_WIDGET (list),  
  			  "drag-motion", 
  			  G_CALLBACK (contact_list_drag_motion), 
@@ -1454,11 +1421,12 @@ contact_list_drag_data_received (GtkWidget         *widget,
 	if (!is_row) {
 		if (g_list_length (groups) != 1) {
 			/* if they have dragged a contact out of a
-			   group then we would set the contact to have
-			   NO groups but only if they were ONE group
-			   to begin with - should we do this
-			   regardless to how many groups they are in
-			   already or not at all? */ 
+			 * group then we would set the contact to have
+			 * NO groups but only if they were ONE group
+			 * to begin with - should we do this
+			 * regardless to how many groups they are in
+			 * already or not at all?  
+			 */ 
 			return;
 		}
 
@@ -1477,8 +1445,8 @@ contact_list_drag_data_received (GtkWidget         *widget,
 			return;
 		}
 
-		/* get source group information */
-		priv = GOSSIP_CONTACT_LIST (widget)->priv;
+		/* Get source group information. */
+		priv = GET_PRIV (widget);
 		if (!priv->drag_row) {
 			return;
 		}
@@ -1499,7 +1467,7 @@ contact_list_drag_data_received (GtkWidget         *widget,
 			drag_del = TRUE;
 		}
 
-		/* create new groups GList */
+		/* Create new groups GList. */
 		for (l = groups, new_groups = NULL; l && drag_success; l = l->next) {
 			gchar *str;
 			
@@ -1610,7 +1578,7 @@ contact_list_drag_begin (GtkWidget      *widget,
 	GtkTreePath           *path;
 	GtkTreeIter            iter;
 
-	priv = GOSSIP_CONTACT_LIST (widget)->priv;
+	priv = GET_PRIV (widget);
 	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -1620,8 +1588,6 @@ contact_list_drag_begin (GtkWidget      *widget,
 	path = gtk_tree_model_get_path (model, &iter);
 	priv->drag_row = gtk_tree_row_reference_new (model, path);
 	gtk_tree_path_free (path);
-
-	/* FIXME: Set a nice icon */
 }
 
 static void
@@ -1639,7 +1605,7 @@ contact_list_drag_data_get (GtkWidget             *widget,
 	GossipContact         *contact;
 	const gchar           *id;
 	
-	priv = GOSSIP_CONTACT_LIST (widget)->priv;
+	priv = GET_PRIV (widget);
 	
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 	if (!priv->drag_row) {
@@ -1684,7 +1650,7 @@ contact_list_drag_end (GtkWidget      *widget,
 {
 	GossipContactListPriv *priv;
 
-	priv = GOSSIP_CONTACT_LIST (widget)->priv;
+	priv = GET_PRIV (widget);
 
 	if (priv->drag_row) {
 		gtk_tree_row_reference_free (priv->drag_row);
@@ -1712,9 +1678,10 @@ contact_list_cell_set_background (GossipContactList  *list,
 		color = style->bg[GTK_STATE_SELECTED]; 
 		
 		/* Here we take the current theme colour and add it to
-		   the colour for white and average the two. This
-		   gives a colour which is inline with the theme but
-		   slightly whiter. */ 
+		 * the colour for white and average the two. This
+		 * gives a colour which is inline with the theme but
+		 * slightly whiter. 
+		 */ 
 		color.red = (color.red + (style->white).red) / 2;
 		color.green = (color.green + (style->white).green) / 2;
 		color.blue = (color.blue + (style->white).blue) / 2;
@@ -1783,7 +1750,7 @@ contact_list_button_press_event_cb (GossipContactList *list,
 {
 	GossipContactListPriv *priv;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 	
 	if (event->button == 3) {
 		GtkTreePath      *path;
@@ -1826,16 +1793,17 @@ contact_list_button_press_event_cb (GossipContactList *list,
 				
 				factory = priv->item_popup_factory;
 				log_item = gtk_item_factory_get_item (factory,
-								      "/Show Log");
-				log_exists = gossip_log_exists (contact);
+								      "/View Previous Conversations");
+
+				log_exists = gossip_log_exists_for_contact (contact);
 				gtk_widget_set_sensitive (log_item, log_exists);
 
-				/* set up invites menu */
+				/* Set up invites menu. */
 				menu = gossip_chat_invite_contact_menu (contact);
 				g_object_unref (contact);
 
 				invite_item = gtk_item_factory_get_item (factory,
-									 "/Invite to Chat Conference");
+									 "/Invite to Chat Room");
 				invite_sep = gtk_item_factory_get_item (factory,
 									"/sep-invite");
 
@@ -1851,7 +1819,7 @@ contact_list_button_press_event_cb (GossipContactList *list,
 					gtk_widget_hide (invite_item);
 				}
 
-				/* FIXME: disable file transfer until finished */
+				/* FIXME: disable file transfer until finished. */
 				ft_item = gtk_item_factory_get_item (factory,
 								     "/Send File...");
 				gtk_widget_hide (ft_item);
@@ -1861,7 +1829,6 @@ contact_list_button_press_event_cb (GossipContactList *list,
 				gtk_widget_hide (ft_item);
 			}		
 
-			/* show */
 			gtk_item_factory_popup (factory,
 						event->x_root, event->y_root,
 						event->button, event->time);
@@ -1884,7 +1851,7 @@ contact_list_row_activated_cb (GossipContactList *list,
 	GossipContact         *contact;
 	GtkTreeIter            iter;
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
@@ -1894,7 +1861,7 @@ contact_list_row_activated_cb (GossipContactList *list,
 			    -1);
 
 	if (!contact) { 
-		/* This is a group */
+		/* This is a group. */
 		return;
 	}
 
@@ -1943,7 +1910,7 @@ contact_list_sort_func (GtkTreeModel *model,
 			    COL_CONTACT, &contact_b,
 			    -1);
 	
-	/* If contact is NULL it means it's a group */
+	/* If contact is NULL it means it's a group. */
 
 	if (!contact_a && contact_b) {
 		ret_val = 1;
@@ -2028,8 +1995,10 @@ contact_list_find_contact_foreach (GtkTreeModel *model,
 		fc->iters = g_list_append (fc->iters, gtk_tree_iter_copy (iter));
 	}
 	
-	/* we want to find ALL contacts that match, this means if we
-	   have the same contact in 3 groups, all iters should be returned. */
+	/* We want to find ALL contacts that match, this means if we
+	 * have the same contact in 3 groups, all iters should be
+	 * returned. 
+	 */
 	return FALSE;
 }
 
@@ -2093,8 +2062,6 @@ contact_list_item_menu_rename_cb (gpointer   data,
 	
 	g_free (str);
 
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	
 	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label,
 		      "use-markup", TRUE,
 		      NULL);
@@ -2178,7 +2145,7 @@ contact_list_item_menu_log_cb (gpointer   data,
                 return;
         }
 
-        gossip_log_show (GTK_WIDGET (data), contact);
+        gossip_log_window_show (contact, NULL);
 
 	g_object_unref (contact);
 }
@@ -2236,7 +2203,7 @@ contact_list_item_menu_remove_cb (gpointer   data,
 	}
 
 	g_object_unref (contact);
-}
+} 
 
 static void
 contact_list_group_menu_rename_cb (gpointer   data,
@@ -2259,7 +2226,7 @@ contact_list_group_menu_rename_cb (gpointer   data,
 
 	str = g_strdup_printf ("<b>%s</b>", group);
 
-	/* translator: %s denotes the contact ID */
+	/* Translator: %s denotes the contact ID */
 	dialog = gtk_message_dialog_new (GTK_WINDOW (gossip_app_get_window ()),
 					 0,
 					 GTK_MESSAGE_QUESTION,
@@ -2269,8 +2236,6 @@ contact_list_group_menu_rename_cb (gpointer   data,
 	
 	g_free (str);
 
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	
 	g_object_set (GTK_MESSAGE_DIALOG (dialog)->label,
 		      "use-markup", TRUE,
 		      NULL);
@@ -2350,7 +2315,7 @@ contact_list_flash_timeout_func (FlashTimeoutData *t_data)
 	GList                 *l, *iters;
 	
 	list = t_data->list;
-	priv = list->priv;
+	priv = GET_PRIV (list);
 	
 	contact = t_data->contact;
 	
@@ -2404,7 +2369,7 @@ contact_list_event_added_cb (GossipEventManager *manager,
 	FlashData             *data;
 	FlashTimeoutData      *t_data;
 	
-	priv = list->priv;
+	priv = GET_PRIV (list);
 	
 	if (gossip_event_get_type (event) != GOSSIP_EVENT_NEW_MESSAGE) {
 		return;
@@ -2415,7 +2380,7 @@ contact_list_event_added_cb (GossipEventManager *manager,
 	data = g_hash_table_lookup (priv->flash_table, contact);
 
 	if (data) {
-		/* Already flashing this item */
+		/* Already flashing this item. */
 		return;
 	}
 
@@ -2454,14 +2419,14 @@ contact_list_event_removed_cb (GossipEventManager *manager,
 		return;
 	}
 	
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 	contact = gossip_message_get_sender (message);
 
 	data = g_hash_table_lookup (priv->flash_table, contact);
 	if (!data) {
-		/* Not flashing this contact */
+		/* Not flashing this contact. */
 		return;
 	}
 
@@ -2519,7 +2484,7 @@ gossip_contact_list_get_selected (GossipContactList *list)
 	
 	g_return_val_if_fail (GOSSIP_IS_CONTACT_LIST (list), NULL);
 	
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
         if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -2545,7 +2510,7 @@ gossip_contact_list_get_selected_group (GossipContactList *list)
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT_LIST (list), NULL);
 	
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
         if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
@@ -2584,12 +2549,12 @@ gossip_contact_list_set_show_offline (GossipContactList *list,
 		
 	g_return_if_fail (GOSSIP_IS_CONTACT_LIST (list));
 
-	priv = list->priv;
+	priv = GET_PRIV (list);
 
 	priv->show_offline = show_offline;
 	show_active = priv->show_active;
 
-	/* disable temporarily */
+	/* Disable temporarily. */
 	priv->show_active = FALSE;
 
 	session = gossip_app_get_session ();
@@ -2603,6 +2568,6 @@ gossip_contact_list_set_show_offline (GossipContactList *list,
 							  list);
 	}
 
-	/* restore to original setting */
+	/* Restore to original setting. */
 	priv->show_active = show_active;
 }
