@@ -40,13 +40,20 @@
 #include "gossip-sound.h"
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
+#include "gossip-log.h"
+
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_GROUP_CHAT, GossipGroupChatPriv))
+
+#define DEBUG_MSG(x)
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
-
 struct _GossipGroupChatPriv {
-	GossipChatroomProvider *provider;
-	GossipChatroomId        room_id;
+	GossipContact          *own_contact;
+
+	GossipChatroomProvider *chatroom_provider;
+	GossipChatroomId        chatroom_id;
 
 	GtkWidget              *widget;
 	GtkWidget              *textview_sw;
@@ -59,7 +66,6 @@ struct _GossipGroupChatPriv {
 
 	GCompletion            *completion;
 
-	GossipContact          *own_contact;
 	GHashTable             *contacts;
 	GList                  *private_chats;
 
@@ -67,7 +73,6 @@ struct _GossipGroupChatPriv {
 	gboolean                contacts_visible;
 };
 
-
 typedef struct {
 	GossipGroupChat *chat;
 	GossipContact   *contact;
@@ -75,14 +80,12 @@ typedef struct {
 	GtkTreeIter      found_iter;
 } ContactFindData;
 
-
 typedef struct {
 	GossipGroupChat *chat;
 	GossipContact   *contact;
 	GtkWidget       *entry;
 } ChatInviteData;
 
-
 static void           group_chat_contact_list_iface_init     (GossipContactListIfaceClass  *iface);
 static void           group_chat_finalize                    (GObject                      *object);
 static void           group_chats_init                       (void);
@@ -111,14 +114,14 @@ static gint           group_chat_contacts_sort_func          (GtkTreeModel
 							      GtkTreeIter                  *iter_b,
 							      gpointer                      user_data);
 static void           group_chat_contacts_setup              (GossipGroupChat              *chat);
-static gboolean       group_chat_contact_find_foreach        (GtkTreeModel                 *model,
+static gboolean       group_chat_contacts_find_foreach       (GtkTreeModel                 *model,
 							      GtkTreePath                  *path,
 							      GtkTreeIter                  *iter,
 							      ContactFindData              *data);
-static gboolean       group_chat_contact_find                (GossipGroupChat              *chat,
+static gboolean       group_chat_contacts_find                (GossipGroupChat              *chat,
 							      GossipContact                *contact,
 							      GtkTreeIter                  *iter);
-static gint           group_chat_contact_completion_func     (const gchar                  *s1,
+static gint           group_chat_contacts_completion_func    (const gchar                  *s1,
 							      const gchar                  *s2,
 							      gsize                         n);
 static void           group_chat_connected_cb                (GossipSession                *session,
@@ -126,17 +129,21 @@ static void           group_chat_connected_cb                (GossipSession
 static void           group_chat_disconnected_cb             (GossipSession                *session,
 							      GossipGroupChat              *chat);
 static void           group_chat_create_gui                  (GossipGroupChat              *chat);
+static void           group_chat_joined_cb                   (GossipChatroomProvider       *provider,
+							      gint                          id,
+							      GossipGroupChat              *chat);
 static void           group_chat_new_message_cb              (GossipChatroomProvider       *provider,
 							      gint                          id,
 							      GossipMessage                *message,
 							      GossipGroupChat              *chat);
-static void           group_chat_new_room_event_cb           (GossipChatroomProvider       *provider,
+static void           group_chat_new_event_cb                (GossipChatroomProvider       *provider,
 							      gint                          id,
 							      const gchar                  *event,
 							      GossipGroupChat              *chat);
-static void           group_chat_title_changed_cb            (GossipChatroomProvider       *provider,
+static void           group_chat_topic_changed_cb            (GossipChatroomProvider       *provider,
 							      gint                          id,
-							      const gchar                  *title,
+							      GossipContact                *who,
+							      const gchar                  *new_topic,
 							      GossipGroupChat              *chat);
 static void           group_chat_contact_joined_cb           (GossipChatroomProvider       *provider,
 							      gint                          id,
@@ -182,8 +189,6 @@ static gboolean       group_chat_get_show_contacts           (GossipChat
 static void           group_chat_set_show_contacts           (GossipChat                   *chat,
 							      gboolean                      show);
 
-
-
 enum {
 	COL_STATUS,
 	COL_NAME,
@@ -191,32 +196,32 @@ enum {
 	NUMBER_OF_COLS
 };
 
-
 enum DndDragType {
 	DND_DRAG_TYPE_CONTACT_ID,
 };
 
-
 static GtkTargetEntry drop_types[] = {
 	{ "text/contact-id", 0, DND_DRAG_TYPE_CONTACT_ID },
 };
 
+static GHashTable   *group_chats = NULL;
+static GObjectClass *parent_class = NULL;
 
 G_DEFINE_TYPE_WITH_CODE (GossipGroupChat, gossip_group_chat, 
 			 GOSSIP_TYPE_CHAT,
 			 G_IMPLEMENT_INTERFACE (GOSSIP_TYPE_CONTACT_LIST_IFACE,
 						group_chat_contact_list_iface_init));
 
-
-static GHashTable *group_chats = NULL;
-
-
 static void
 gossip_group_chat_class_init (GossipGroupChatClass *klass)
 {
-        GObjectClass    *object_class = G_OBJECT_CLASS (klass);
-	GossipChatClass *chat_class   = GOSSIP_CHAT_CLASS (klass);
-                                                                                
+        GObjectClass    *object_class;
+	GossipChatClass *chat_class;
+
+	object_class = G_OBJECT_CLASS (klass);
+	parent_class = g_type_class_peek_parent (klass);
+	chat_class = GOSSIP_CHAT_CLASS (klass);
+            
         object_class->finalize = group_chat_finalize;
 
 	chat_class->get_name          = group_chat_get_name;
@@ -229,17 +234,17 @@ gossip_group_chat_class_init (GossipGroupChatClass *klass)
 	chat_class->get_group_chat    = group_chat_get_group_chat;
 	chat_class->get_show_contacts = group_chat_get_show_contacts;
 	chat_class->set_show_contacts = group_chat_set_show_contacts;
+
+	g_type_class_add_private (object_class, sizeof (GossipGroupChatPriv));
 }
 
 static void
 gossip_group_chat_init (GossipGroupChat *chat)
 {
-        GossipGroupChatPriv *priv;
-                                                                                
-        priv = g_new0 (GossipGroupChatPriv, 1);
-                                                                                
-        chat->priv = priv;
+	GossipGroupChatPriv *priv;
 
+	priv = GET_PRIV (chat);
+                                                                               
 	priv->contacts_visible = TRUE;
 }
 
@@ -256,26 +261,28 @@ group_chat_finalize (GObject *object)
 	GossipGroupChatPriv *priv;
 
 	chat = GOSSIP_GROUP_CHAT (object);
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	g_signal_handlers_disconnect_by_func (priv->provider, 
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider, 
+					      group_chat_joined_cb, chat);
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider, 
 					      group_chat_new_message_cb, chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
-					      group_chat_new_room_event_cb, 
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
+					      group_chat_new_event_cb, 
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
-					      group_chat_title_changed_cb, 
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
+					      group_chat_topic_changed_cb, 
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_contact_joined_cb,
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_contact_left_cb,
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_contact_presence_updated_cb,
 					      chat);
-	g_signal_handlers_disconnect_by_func (priv->provider,
+	g_signal_handlers_disconnect_by_func (priv->chatroom_provider,
 					      group_chat_contact_updated_cb,
 					      chat);
 	
@@ -289,8 +296,8 @@ group_chat_finalize (GObject *object)
 	if (priv->own_contact) {
 		g_object_unref (priv->own_contact);
 	}
-	
-	g_free (priv);
+
+/* 	G_OBJECT_CLASS (parent_class)->finalize (object); */
 }
 
 static void
@@ -325,17 +332,25 @@ group_chat_key_press_event_cb (GtkWidget       *widget,
 	GList               *list, *l, *completed_list;
 	gboolean  is_start_of_buffer;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	/* Catch enter but not ctrl/shift-enter */
 	if (IS_ENTER (event->keyval) && !(event->state & GDK_SHIFT_MASK)) {
+		GossipChat   *parent;
+		GtkTextView  *view;
+		GtkIMContext *context;
+
 		/* This is to make sure that kinput2 gets the enter. And if
                  * it's handled there we shouldn't send on it. This is because
                  * kinput2 uses Enter to commit letters. See:
                  * http://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=104299
                  */
-		if (gtk_im_context_filter_keypress (GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view)->im_context, event)) {
-			GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view)->need_im_reset = TRUE;
+		parent = GOSSIP_CHAT (chat);
+		view = GTK_TEXT_VIEW (parent->input_text_view);
+		context = view->im_context;
+
+		if (gtk_im_context_filter_keypress (context, event)) {
+			GTK_TEXT_VIEW (view)->need_im_reset = TRUE;
 			return TRUE;
 		}
 		
@@ -347,13 +362,15 @@ group_chat_key_press_event_cb (GtkWidget       *widget,
 		/* Newline for shift-enter. */
 		return FALSE;
 	}
-	else if (event->keyval == GDK_Page_Up) {
+	else if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
+		 event->keyval == GDK_Page_Up) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->textview_sw));
 		gtk_adjustment_set_value (adj, adj->value - adj->page_size);
 
 		return TRUE;
 	}
-	else if (event->keyval == GDK_Page_Down) {
+	else if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
+		 event->keyval == GDK_Page_Down) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->textview_sw));
 		val = MIN (adj->value + adj->page_size, adj->upper - adj->page_size);
 		gtk_adjustment_set_value (adj, val);
@@ -440,7 +457,7 @@ group_chat_focus_in_event_cb (GtkWidget       *widget,
 {
 	GossipGroupChatPriv *priv;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
 
@@ -462,7 +479,7 @@ group_chat_drag_data_received (GtkWidget        *widget,
 	const gchar         *id;
 	gchar               *str;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
 	id = (const gchar*) selection->data;
 	g_print ("Received drag & drop contact from roster with id:'%s'\n", id);
@@ -480,7 +497,7 @@ group_chat_drag_data_received (GtkWidget        *widget,
  	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
 	g_free (str);
 
-	gossip_chat_invite_dialog (contact, priv->room_id);
+	gossip_chat_invite_dialog (contact, priv->chatroom_id);
 
 	/* clean up dnd */
 	gtk_drag_finish (context, TRUE, FALSE, GDK_CURRENT_TIME);
@@ -516,7 +533,7 @@ group_chat_widget_destroy_cb (GtkWidget       *widget,
 {
 	GossipGroupChatPriv *priv;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	/* disconnect signals */
 	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
@@ -527,9 +544,9 @@ group_chat_widget_destroy_cb (GtkWidget       *widget,
 					      group_chat_disconnected_cb,
 					      chat);
 
-	gossip_chatroom_provider_leave (priv->provider, priv->room_id);
+	gossip_chatroom_provider_leave (priv->chatroom_provider, priv->chatroom_id);
 	
-	g_hash_table_remove (group_chats, GINT_TO_POINTER (priv->room_id));
+	g_hash_table_remove (group_chats, GINT_TO_POINTER (priv->chatroom_id));
 }
 
 static gint
@@ -562,7 +579,8 @@ group_chat_contacts_setup (GossipGroupChat *chat)
 	GtkCellRenderer     *cell;	
 	GtkTreeViewColumn   *col;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
+
 	tree = GTK_TREE_VIEW (priv->treeview);
 	tv = GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->view);
 
@@ -605,10 +623,10 @@ group_chat_contacts_setup (GossipGroupChat *chat)
 }
 
 static gboolean
-group_chat_contact_find_foreach (GtkTreeModel    *model,
-				 GtkTreePath     *path,
-				 GtkTreeIter     *iter,
-				 ContactFindData *data)
+group_chat_contacts_find_foreach (GtkTreeModel    *model,
+				  GtkTreePath     *path,
+				  GtkTreeIter     *iter,
+				  ContactFindData *data)
 {
 	GossipContact *contact;
 	gboolean       equal;
@@ -630,15 +648,15 @@ group_chat_contact_find_foreach (GtkTreeModel    *model,
 }
 
 static gboolean
-group_chat_contact_find (GossipGroupChat *chat,
-			 GossipContact   *contact, 
-			 GtkTreeIter     *iter)
+group_chat_contacts_find (GossipGroupChat *chat,
+			  GossipContact   *contact, 
+			  GtkTreeIter     *iter)
 {
 	GossipGroupChatPriv *priv;
 	ContactFindData      data;
 	GtkTreeModel        *model;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	data.found = FALSE;
 	data.contact = contact;
@@ -647,7 +665,7 @@ group_chat_contact_find (GossipGroupChat *chat,
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
 	gtk_tree_model_foreach (model,
-				(GtkTreeModelForeachFunc) group_chat_contact_find_foreach,
+				(GtkTreeModelForeachFunc) group_chat_contacts_find_foreach,
 				&data);
 
 	if (data.found) {
@@ -658,7 +676,7 @@ group_chat_contact_find (GossipGroupChat *chat,
 }
 
 static gint
-group_chat_contact_completion_func (const gchar *s1, 
+group_chat_contacts_completion_func (const gchar *s1, 
 				    const gchar *s2,
 				    gsize        n)
 {
@@ -687,8 +705,6 @@ group_chat_connected_cb (GossipSession   *session,
 {
  	g_return_if_fail (GOSSIP_IS_GROUP_CHAT (chat)); 
 
- 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, TRUE); 
-
  	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, _("Connected")); 
 }
 
@@ -712,7 +728,7 @@ group_chat_create_gui (GossipGroupChat *chat)
 	GtkWidget           *input_textview_sw;
 	GList               *list;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	glade = gossip_glade_get_file (GLADEDIR "/group-chat.glade",
 				       "group_chat_widget",
@@ -775,27 +791,64 @@ group_chat_create_gui (GossipGroupChat *chat)
 	
 	priv->completion = g_completion_new (NULL);
 	g_completion_set_compare (priv->completion,
-				  group_chat_contact_completion_func);
+				  group_chat_contacts_completion_func);
 		
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
 	group_chat_contacts_setup (chat);
 }
 
+static void 
+group_chat_joined_cb (GossipChatroomProvider *provider,
+		      gint                    id,
+		      GossipGroupChat        *chat)
+{
+	GossipGroupChatPriv *priv;
+	GossipChatView      *chatview;
+	GtkTreeView         *view;
+	GtkTreeModel        *model;
+	GtkListStore        *store;
+	
+	priv = GET_PRIV (chat);
+
+	if (priv->chatroom_id != id) {
+		return;
+	}
+
+	DEBUG_MSG (("GroupChat: [%d] Joined", id));
+
+	chatview = GOSSIP_CHAT (chat)->view;
+
+	/* Clear previous messages */
+	gossip_chat_view_clear (chatview);
+
+	/* Clear previous roster */
+	view = GTK_TREE_VIEW (priv->treeview);
+	model = gtk_tree_view_get_model (view);
+	store = GTK_LIST_STORE (model);
+	gtk_list_store_clear (store);
+
+	/* Allow use of the input textview */
+ 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, TRUE); 
+}
+
 static void 
 group_chat_new_message_cb (GossipChatroomProvider *provider,
 			   gint                    id,
 			   GossipMessage          *message,
 			   GossipGroupChat        *chat)
 {
-	GossipGroupChatPriv *priv;
-	const gchar         *invite;
+	GossipGroupChatPriv  *priv;
+	GossipChatroom       *chatroom;
+	GossipChatroomInvite *invite;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
+	DEBUG_MSG (("GroupChat: [%d] New message", id));
+
 	invite = gossip_message_get_invite (message);
 	if (invite) {
 		gossip_chat_view_append_invite (GOSSIP_CHAT (chat)->view,
@@ -822,41 +875,61 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		gossip_sound_play (GOSSIP_SOUND_CHAT);
 	}
 
-	g_signal_emit_by_name (chat, "new-message");
+ 	chatroom = gossip_chatroom_provider_find (priv->chatroom_provider, 
+						  priv->chatroom_id);
+	gossip_log_message_for_chatroom (chatroom, message, FALSE);
+
+	g_signal_emit_by_name (chat, "new-message", message);
 }
 
 static void 
-group_chat_new_room_event_cb (GossipChatroomProvider *provider,
-			      gint                    id,
-			      const gchar            *event,
-			      GossipGroupChat        *chat)
+group_chat_new_event_cb (GossipChatroomProvider *provider,
+			 gint                    id,
+			 const gchar            *event,
+			 GossipGroupChat        *chat)
 {
 	GossipGroupChatPriv *priv;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
+	DEBUG_MSG (("GroupChat: [%d] New event:'%s'", id, event));
+
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, event);
 }
 
 static void
-group_chat_title_changed_cb (GossipChatroomProvider *provider,
+group_chat_topic_changed_cb (GossipChatroomProvider *provider,
 			     gint                    id,
-			     const gchar            *title,
+			     GossipContact          *who,
+			     const gchar            *new_topic,
 			     GossipGroupChat        *chat)
 {
 	GossipGroupChatPriv *priv;
+	gchar               *event;
+	gchar               *str;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
+
+	DEBUG_MSG (("GroupChat: [%d] Topic changed by:'%s' to:'%s'", 
+		    id, gossip_contact_get_id (who), new_topic));
 	
-	gtk_entry_set_text (GTK_ENTRY (priv->topic_entry), title);
+	gtk_entry_set_text (GTK_ENTRY (priv->topic_entry), new_topic);
+	
+	str = g_strdup_printf (_("%s has set the topic"),
+				 gossip_contact_get_name (who));
+	event = g_strconcat (str, ": ", new_topic, NULL);
+	g_free (str);
+
+	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, event);
+	g_free (event);
 }
 
 static void
@@ -870,20 +943,15 @@ group_chat_contact_joined_cb (GossipChatroomProvider *provider,
 	GtkTreeIter          iter;
 	GdkPixbuf           *pixbuf;
 		
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
-	if (gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_USER) {
-		if (priv->own_contact) {
-			g_object_unref (priv->own_contact);
-		}
+	DEBUG_MSG (("GroupChat: [%d] Contact joined:'%s'", 
+		    id, gossip_contact_get_id (contact)));
 
-		priv->own_contact = g_object_ref (contact);
-	}
-	
 	pixbuf = gossip_pixbuf_for_contact (contact);
 	
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
@@ -909,14 +977,16 @@ group_chat_contact_left_cb (GossipChatroomProvider *provider,
 	GossipGroupChatPriv *priv;
 	GtkTreeIter          iter;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
+	DEBUG_MSG (("GroupChat: [%d] Contact left:'%s'", 
+		    id, gossip_contact_get_id (contact)));
 	
-	if (group_chat_contact_find (chat, contact, &iter)) {
+	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GtkTreeModel *model;
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
@@ -934,13 +1004,13 @@ group_chat_contact_presence_updated_cb (GossipChatroomProvider *provider,
 	GossipGroupChatPriv *priv;
 	GtkTreeIter          iter;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
-	if (group_chat_contact_find (chat, contact, &iter)) {
+	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GdkPixbuf    *pixbuf;
 		GtkTreeModel *model;
 		
@@ -968,13 +1038,13 @@ group_chat_contact_updated_cb (GossipChatroomProvider *provider,
 	GossipGroupChatPriv *priv;
 	GtkTreeIter          iter;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	if (priv->room_id != id) {
+	if (priv->chatroom_id != id) {
 		return;
 	}
 
-	if (group_chat_contact_find (chat, contact, &iter)) {
+	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GtkTreeModel *model;
 		
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
@@ -993,10 +1063,10 @@ group_chat_topic_activate_cb (GtkEntry *entry,
 {
 	GossipGroupChatPriv *priv;
 	
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
-	gossip_chatroom_provider_change_topic (priv->provider, 
-					       priv->room_id,
+	gossip_chatroom_provider_change_topic (priv->chatroom_provider, 
+					       priv->chatroom_id,
 					       gtk_entry_get_text (entry));
 	
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
@@ -1011,7 +1081,7 @@ group_chat_private_chat_new (GossipGroupChat *chat,
 	GossipChatManager   *chat_manager;
 	GtkWidget           *widget;
 
- 	priv = chat->priv; 
+ 	priv = GET_PRIV (chat); 
 
 	chat_manager = gossip_app_get_chat_manager ();
 	gossip_chat_manager_show_chat (chat_manager, contact);
@@ -1024,9 +1094,10 @@ group_chat_private_chat_new (GossipGroupChat *chat,
 			   chat);
 
 	/* Set private chat sensitive, since previously we might have
-	   already had a private chat with this JID and in this room,
-	   and they would have been made insensitive when the last
-	   room was closed */
+	 * already had a private chat with this JID and in this room,
+	 * and they would have been made insensitive when the last
+	 * room was closed.
+	 */
 	widget = gossip_chat_get_widget (GOSSIP_CHAT (private_chat));
 	gtk_widget_set_sensitive (widget, TRUE);
 }
@@ -1037,7 +1108,7 @@ group_chat_private_chat_removed (GossipGroupChat *chat,
 {
 	GossipGroupChatPriv *priv;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	priv->private_chats = g_list_remove (priv->private_chats, private_chat);
 }
@@ -1065,7 +1136,7 @@ group_chat_send (GossipGroupChat *chat)
 	gchar		    *msg;
 	gboolean             handled_command = FALSE;
 
-	priv = chat->priv;
+	priv = GET_PRIV (chat);
 	
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view));
 
@@ -1080,8 +1151,8 @@ group_chat_send (GossipGroupChat *chat)
 		const gchar *nick;
 
 		nick = msg + 6;
-		gossip_chatroom_provider_change_nick (priv->provider,
-						      priv->room_id,
+		gossip_chatroom_provider_change_nick (priv->chatroom_provider,
+						      priv->chatroom_id,
 						      nick);
 		handled_command = TRUE;
 	}
@@ -1089,8 +1160,8 @@ group_chat_send (GossipGroupChat *chat)
 		const gchar *topic;
 
 		topic = msg + 7;
-		gossip_chatroom_provider_change_topic (priv->provider,
-						       priv->room_id,
+		gossip_chatroom_provider_change_topic (priv->chatroom_provider,
+						       priv->chatroom_id,
 						       topic);
 		handled_command = TRUE;
 	}
@@ -1104,7 +1175,7 @@ group_chat_send (GossipGroupChat *chat)
 	}
 
 	if (!handled_command) {
-		gossip_chatroom_provider_send (priv->provider, priv->room_id, msg);
+		gossip_chatroom_provider_send (priv->chatroom_provider, priv->chatroom_id, msg);
 		
 		gossip_app_force_non_away ();
 	}
@@ -1137,11 +1208,10 @@ group_chat_get_nick_list (GossipGroupChat *chat)
 {
 	GossipGroupChatPriv *priv;
 	GtkTreeModel        *model;
-	GList               *list;
+	GList               *list = NULL;
+
+	priv = GET_PRIV (chat);
 
-	priv = chat->priv;
-	list = NULL;
-	
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 	
 	gtk_tree_model_foreach (model,
@@ -1154,13 +1224,13 @@ group_chat_get_nick_list (GossipGroupChat *chat)
 static GtkWidget *
 group_chat_get_widget (GossipChat *chat)
 {
-	GossipGroupChat     *g_chat;
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL);
 
-	g_chat = GOSSIP_GROUP_CHAT (chat);
-	priv   = g_chat->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 
 	return priv->widget;
 }
@@ -1168,13 +1238,13 @@ group_chat_get_widget (GossipChat *chat)
 static const gchar *
 group_chat_get_name (GossipChat *chat)
 {
-	GossipGroupChat     *g_chat;
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL);
 
-	g_chat = GOSSIP_GROUP_CHAT (chat);
-	priv   = g_chat->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 
 	return priv->name;
 }
@@ -1182,13 +1252,13 @@ group_chat_get_name (GossipChat *chat)
 static gchar *
 group_chat_get_tooltip (GossipChat *chat)
 {
-	GossipGroupChat     *g_chat;
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL);
 
-	g_chat = GOSSIP_GROUP_CHAT (chat);
-	priv   = g_chat->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 
 	return g_strdup (priv->name);
 }
@@ -1196,9 +1266,20 @@ group_chat_get_tooltip (GossipChat *chat)
 static GdkPixbuf *
 group_chat_get_status_pixbuf (GossipChat *chat)
 {
-	static GdkPixbuf *pixbuf = NULL;
+	GossipGroupChat     *group_chat;
+	GossipGroupChatPriv *priv;
+	GossipChatroom      *chatroom;
+	GdkPixbuf           *pixbuf = NULL;
 
-	if (pixbuf == NULL) {
+	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL);
+
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+ 	priv = GET_PRIV (group_chat); 
+
+ 	chatroom = gossip_chatroom_provider_find (priv->chatroom_provider, priv->chatroom_id);
+	pixbuf = gossip_pixbuf_for_chatroom_status (chatroom, GTK_ICON_SIZE_MENU);
+
+	if (!pixbuf) {
 		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_GROUP_MESSAGE,
 						   GTK_ICON_SIZE_MENU);
 	}
@@ -1217,13 +1298,13 @@ group_chat_get_contact (GossipChat *chat)
 static GossipContact *
 group_chat_get_own_contact (GossipChat *chat)
 {
-	GossipGroupChat     *g_chat;
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL);
 
-	g_chat = GOSSIP_GROUP_CHAT (chat);
-	priv   = g_chat->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 
 	return priv->own_contact;
 }
@@ -1248,11 +1329,13 @@ group_chat_get_group_chat (GossipChat *chat)
 static gboolean
 group_chat_get_show_contacts (GossipChat *chat)
 {
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), FALSE);
 
-	priv = GOSSIP_GROUP_CHAT (chat)->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 	
 	return priv->contacts_visible;
 }
@@ -1261,11 +1344,13 @@ static void
 group_chat_set_show_contacts (GossipChat *chat, 
 			      gboolean    show)
 {
+	GossipGroupChat     *group_chat;
 	GossipGroupChatPriv *priv;
 
 	g_return_if_fail (GOSSIP_IS_GROUP_CHAT (chat));
 
-	priv = GOSSIP_GROUP_CHAT (chat)->priv;
+	group_chat = GOSSIP_GROUP_CHAT (chat);
+	priv = GET_PRIV (group_chat);
 	
 	priv->contacts_visible = show;
 
@@ -1279,27 +1364,62 @@ group_chat_set_show_contacts (GossipChat *chat,
 	}
 }
 
+/* Scroll down after the back-log has been received. */
+static gboolean
+group_chat_scroll_down_func (GossipChat *chat)
+{
+	gossip_chat_scroll_down (chat);
+ 	g_object_unref (chat);
+
+	return FALSE;
+}
+
 GossipGroupChat *
-gossip_group_chat_show (GossipChatroomProvider *provider,
-			GossipChatroomId        id)
+gossip_group_chat_new (GossipChatroomProvider *provider,
+		       GossipChatroomId        id)
 {
 	GossipGroupChat     *chat;
 	GossipGroupChatPriv *priv;
+	GossipContact       *own_contact;
+ 	gchar               *own_contact_id;
+	GossipChatroom      *chatroom;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM_PROVIDER (provider), NULL);
 	
 	group_chats_init ();
-
+	
+	/* Check this group chat is not already shown */
 	chat = g_hash_table_lookup (group_chats, GINT_TO_POINTER (id));
 	if (chat) {
 		gossip_chat_present (GOSSIP_CHAT (chat));
 		return chat;
 	}
 
-	chat = g_object_new (GOSSIP_TYPE_GROUP_CHAT, NULL);
-	priv = chat->priv;
+	/* Get important details like own contact, etc */
+	chatroom = gossip_chatroom_provider_find (provider, id);
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
 
-	priv->room_id = id;
+	own_contact_id = g_strdup_printf ("%s/%s", 
+					  gossip_chatroom_get_id_str (chatroom),
+					  gossip_chatroom_get_nick (chatroom));
 	
-	priv->name = g_strdup (gossip_chatroom_provider_get_room_name (provider, id));
+	own_contact = gossip_contact_new_full (GOSSIP_CONTACT_TYPE_TEMPORARY, 
+					       gossip_chatroom_get_account (chatroom),
+					       own_contact_id,
+					       gossip_chatroom_get_nick (chatroom));
+	g_free (own_contact_id);
+
+	/* Create new group chat object */
+	chat = g_object_new (GOSSIP_TYPE_GROUP_CHAT, NULL);
+
+	priv = GET_PRIV (chat);
+
+	priv->own_contact = own_contact;
+
+	priv->chatroom_id = id;
+	priv->chatroom_provider = provider;
+
+	priv->name = g_strdup (gossip_chatroom_get_name (chatroom));
 
 	priv->private_chats = NULL;
 	
@@ -1307,14 +1427,17 @@ gossip_group_chat_show (GossipChatroomProvider *provider,
 
 	g_hash_table_insert (group_chats, GINT_TO_POINTER (id), chat);
 
+	g_signal_connect (provider, "chatroom-joined",
+			  G_CALLBACK (group_chat_joined_cb),
+			  chat);
 	g_signal_connect (provider, "chatroom-new-message",
 			  G_CALLBACK (group_chat_new_message_cb),
 			  chat);
-	g_signal_connect (provider, "chatroom-new-room-event",
-			  G_CALLBACK (group_chat_new_room_event_cb),
+	g_signal_connect (provider, "chatroom-new-event",
+			  G_CALLBACK (group_chat_new_event_cb),
 			  chat);
-	g_signal_connect (provider, "chatroom-title-changed",
-			  G_CALLBACK (group_chat_title_changed_cb),
+	g_signal_connect (provider, "chatroom-topic-changed",
+			  G_CALLBACK (group_chat_topic_changed_cb),
 			  chat);
 	g_signal_connect (provider, "chatroom-contact-joined",
 			  G_CALLBACK (group_chat_contact_joined_cb),
@@ -1339,21 +1462,34 @@ gossip_group_chat_show (GossipChatroomProvider *provider,
 				 G_CALLBACK (group_chat_disconnected_cb),
 				 chat, 0);
 
-	priv->provider = provider;
-	
 	gossip_chat_present (GOSSIP_CHAT (chat));
 
+ 	g_object_ref (chat);
+	g_idle_add ((GSourceFunc) group_chat_scroll_down_func, chat);
+	
 	return chat;
 }
 
 GossipChatroomId 
-gossip_group_chat_get_room_id (GossipGroupChat *chat)
+gossip_group_chat_get_chatroom_id (GossipGroupChat *chat)
 {
 	GossipGroupChatPriv *priv;
 
  	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), 0); 
 
-	priv = chat->priv;	
+	priv = GET_PRIV (chat);	
 
-	return priv->room_id;
+	return priv->chatroom_id;
+}
+
+GossipChatroomProvider *
+gossip_group_chat_get_chatroom_provider (GossipGroupChat *chat)
+{
+	GossipGroupChatPriv *priv;
+
+ 	g_return_val_if_fail (GOSSIP_IS_GROUP_CHAT (chat), NULL); 
+
+	priv = GET_PRIV (chat);	
+
+ 	return priv->chatroom_provider;
 }
