@@ -32,7 +32,6 @@
 #include "gossip-preferences.h"
 #include "gossip-spell.h"
 
-
 typedef struct {
 	GossipApp *app;
  	
@@ -49,11 +48,12 @@ typedef struct {
 	GtkWidget *spell_checker_vbox;
 	GtkWidget *spell_checker_checkbutton;
 	GtkWidget *spell_checker_treeview;
+
+	GtkWidget *theme_combobox;
 	
 	GList     *ids;
 } GossipPreferences;
 
-
 enum {
 	COL_LANG_ENABLED,
 	COL_LANG_CODE,
@@ -61,6 +61,11 @@ enum {
 	COL_LANG_COUNT
 };
 
+enum {
+	COL_COMBO_VISIBLE_NAME,
+	COL_COMBO_NAME,
+	COL_COMBO_COUNT
+};
 
 static void     preferences_setup_widgets                (GossipPreferences      *preferences);
 static void     preferences_languages_setup              (GossipPreferences      *preferences);
@@ -78,12 +83,15 @@ static gboolean preferences_languages_load_foreach       (GtkTreeModel
 static void     preferences_languages_cell_toggled_cb    (GtkCellRendererToggle  *cell,
 							  gchar                  *path_string,
 							  GossipPreferences      *preferences);
+static void     preferences_themes_setup                 (GossipPreferences      *preferences);
 static void     preferences_set_boolean_from_gconf       (const gchar            *key,
 							  GtkWidget              *widget);
 static void     preferences_set_int_from_gconf           (const gchar            *key,
 							  GtkWidget              *widget);
 static void     preferences_set_string_from_gconf        (const gchar            *key,
 							  GtkWidget              *widget);
+static void     preferences_set_string_combo_from_gconf  (const gchar            *key,
+							  GtkWidget              *widget);
 static void     preferences_notify_int_cb                (GConfClient            *client,
 							  guint                   cnxn_id,
 							  GConfEntry             *entry,
@@ -92,6 +100,10 @@ static void     preferences_notify_string_cb             (GConfClient
 							  guint                   cnxn_id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
+static void     preferences_notify_string_combo_cb       (GConfClient            *client,
+							  guint                   cnxn_id,
+							  GConfEntry             *entry,
+							  gpointer                user_data);
 static void     preferences_notify_bool_cb               (GConfClient            *client,
 							  guint                   cnxn_id,
 							  GConfEntry             *entry,
@@ -109,6 +121,9 @@ static void     preferences_hookup_entry                 (GossipPreferences
 static void     preferences_hookup_toggle_button         (GossipPreferences      *preferences,
 							  const gchar            *key,
 							  GtkWidget              *widget);
+static void     preferences_hookup_string_combo          (GossipPreferences      *preferences,
+							  const gchar            *key,
+							  GtkWidget              *widget);
 static void     preferences_hookup_sensitivity           (GossipPreferences      *preferences,
 							  const gchar            *key,
 							  GtkWidget              *widget);
@@ -118,14 +133,14 @@ static void     preferences_entry_value_changed_cb       (GtkWidget
 							  gpointer                user_data);
 static void     preferences_toggle_button_toggled_cb     (GtkWidget              *button,
 							  gpointer                user_data);
+static void     preferences_string_combo_changed_cb      (GtkWidget *button,
+							  gpointer                user_data);
 static void     preferences_destroy_cb                   (GtkWidget              *widget,
 							  GossipPreferences      *preferences);
 static void     preferences_response_cb                  (GtkWidget              *widget,
 							  gint                    response,
 							  GossipPreferences      *preferences);
 
-
-
 static void
 preferences_setup_widgets (GossipPreferences *preferences)
 {
@@ -156,6 +171,10 @@ preferences_setup_widgets (GossipPreferences *preferences)
 	preferences_hookup_toggle_button (preferences,
 					  GCONF_PATH "/conversation/enable_spell_checker",
 					  preferences->spell_checker_checkbutton);
+
+	preferences_hookup_string_combo (preferences,
+					 GCONF_PATH "/conversation/theme",
+					 preferences->theme_combobox);
 }
 
 static void 
@@ -188,12 +207,10 @@ preferences_languages_setup (GossipPreferences *preferences)
  			  G_CALLBACK (preferences_languages_cell_toggled_cb),  
  			  preferences);
 	
-	column = gtk_tree_view_column_new_with_attributes (_("Select"), renderer,
+	column = gtk_tree_view_column_new_with_attributes (NULL, renderer,
 							   "active", COL_LANG_ENABLED,
 							   NULL);
 	
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width (column, 50);
 	gtk_tree_view_append_column (view, column);
 	
 	renderer = gtk_cell_renderer_text_new ();
@@ -219,7 +236,6 @@ preferences_languages_add (GossipPreferences *preferences)
 {
 	GtkTreeView  *view;
 	GtkListStore *store;
-
 	GossipSpell  *spell;
 	GList        *codes, *l;
 
@@ -230,13 +246,13 @@ preferences_languages_add (GossipPreferences *preferences)
 	codes = gossip_spell_get_language_codes (spell);
 
 	for (l = codes; l; l = l->next) {
-		GtkTreeIter   iter;
-		const gchar  *code;
-		const gchar  *language;
+		GtkTreeIter  iter;
+		const gchar *code;
+		const gchar *language;
 
 		code = l->data; 
-		language = gossip_spell_get_language_name (code);
-
+		language = gossip_spell_get_language_name (spell, code);
+		
 		if (!language || strlen (language) < 1) {
 			continue;
 		}
@@ -385,14 +401,13 @@ preferences_languages_load_foreach (GtkTreeModel  *model,
 static void 
 preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell, 
 				       gchar                 *path_string, 
-			 GossipPreferences *preferences)
+				       GossipPreferences     *preferences)
 {
 	GtkTreeView  *view;
 	GtkTreeModel *model;
 	GtkListStore *store;
 	GtkTreePath  *path;
 	GtkTreeIter   iter;
-
 	gboolean      enabled;
 
 	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
@@ -412,6 +427,35 @@ preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell,
 	preferences_languages_save (preferences);
 }
 
+static void
+preferences_themes_setup (GossipPreferences *preferences)
+{
+	GtkComboBox     *combo;
+	GtkListStore    *model;
+	GtkTreeIter      iter;
+
+	combo = GTK_COMBO_BOX (preferences->theme_combobox);
+
+	model = gtk_list_store_new (COL_COMBO_COUNT,
+				    G_TYPE_STRING,
+				    G_TYPE_STRING);
+
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter,
+			    COL_COMBO_VISIBLE_NAME, _("Classic"),
+			    COL_COMBO_NAME, "classic",
+			    -1);
+	
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter,
+			    COL_COMBO_VISIBLE_NAME, _("Blue"),
+			    COL_COMBO_NAME, "blue",
+			    -1);
+	
+	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (model));
+	g_object_unref (model);
+}	
+
 static void
 preferences_set_boolean_from_gconf (const gchar *key, GtkWidget *widget)
 {
@@ -444,6 +488,49 @@ preferences_set_string_from_gconf (const gchar *key, GtkWidget *widget)
 	g_free (value);
 }
 
+static void
+preferences_set_string_combo_from_gconf (const gchar *key, GtkWidget *widget)
+{
+	gchar        *value;
+	GtkTreeModel *model; 
+	GtkTreeIter   iter;
+	gboolean      found;
+	
+	value = gconf_client_get_string (gossip_app_get_gconf_client (), key, NULL);
+
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+
+	found = FALSE;
+	if (value && gtk_tree_model_get_iter_first (model, &iter)) {
+		gchar *name;
+		
+		do {
+			gtk_tree_model_get (model, &iter,
+					    COL_COMBO_NAME, &name, 
+					    -1);
+			
+			if (strcmp (name, value) == 0) {
+				found = TRUE;
+				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
+				break;
+			} else {
+				found = FALSE;
+			}
+
+			g_free (name);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+
+	/* Fallback to classic. */
+	if (!found) {
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
+		}
+	}
+
+	g_free (value);
+}
+
 static void
 preferences_notify_int_cb (GConfClient *client,
 			   guint        cnxn_id,
@@ -472,6 +559,15 @@ preferences_notify_string_cb (GConfClient *client,
 			    gconf_value_get_string (value));
 }
 
+static void
+preferences_notify_string_combo_cb (GConfClient *client,
+				    guint        cnxn_id,
+				    GConfEntry  *entry,
+				    gpointer     user_data)
+{
+	preferences_set_string_combo_from_gconf (gconf_entry_get_key (entry), user_data);
+}
+
 static void
 preferences_notify_bool_cb (GConfClient *client,
 			    guint        cnxn_id,
@@ -593,6 +689,34 @@ preferences_hookup_toggle_button (GossipPreferences *preferences,
 					   GUINT_TO_POINTER (id));
 }
 
+static void
+preferences_hookup_string_combo (GossipPreferences *preferences,
+				 const gchar       *key,
+				 GtkWidget         *widget)
+{
+	guint id;
+
+	preferences_set_string_combo_from_gconf (key, widget);
+	
+	g_object_set_data_full (G_OBJECT (widget), "key",
+				g_strdup (key), g_free); 
+	
+	g_signal_connect (widget,
+			  "changed",
+			  G_CALLBACK (preferences_string_combo_changed_cb),
+			  NULL);
+
+	id = gconf_client_notify_add (gossip_app_get_gconf_client (),
+				      key,
+				      preferences_notify_string_combo_cb,
+				      widget,
+				      NULL,
+				      NULL);
+	
+	preferences->ids = g_list_prepend (preferences->ids,
+					   GUINT_TO_POINTER (id));
+}
+
 static void
 preferences_hookup_sensitivity (GossipPreferences *preferences,
 			       const gchar       *key,
@@ -643,7 +767,6 @@ preferences_entry_value_changed_cb (GtkWidget *entry,
 				 NULL);
 }
 
-
 static void
 preferences_toggle_button_toggled_cb (GtkWidget *button,
 				      gpointer   user_data)
@@ -658,6 +781,41 @@ preferences_toggle_button_toggled_cb (GtkWidget *button,
 			       NULL);
 }
 
+static void
+preferences_string_combo_changed_cb (GtkWidget *combo,
+				     gpointer   user_data)
+{
+	const gchar  *key;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	gchar        *name;
+
+	key = g_object_get_data (G_OBJECT (combo), "key");
+
+	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo), &iter)) {
+		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+		
+		gtk_tree_model_get (model, &iter,
+				    COL_COMBO_NAME, &name,
+				    -1);
+		
+		gconf_client_set_string (gossip_app_get_gconf_client (),
+					 key,
+					 name,
+					 NULL);
+
+		g_free (name);
+	}
+}
+
+static void
+preferences_response_cb (GtkWidget         *widget,
+			 gint               response,
+			 GossipPreferences *preferences)
+{
+	gtk_widget_destroy (widget);
+}
+
 static void
 preferences_destroy_cb (GtkWidget         *widget,
 			GossipPreferences *preferences)
@@ -674,15 +832,6 @@ preferences_destroy_cb (GtkWidget         *widget,
 	g_free (preferences);
 }
 
-static void
-preferences_response_cb (GtkWidget         *widget,
-			 gint               response,
-			 GossipPreferences *preferences)
-{
-	gtk_widget_destroy (widget);
-}
-
-
 void
 gossip_preferences_show (void)
 {
@@ -697,24 +846,28 @@ gossip_preferences_show (void)
         preferences = g_new0 (GossipPreferences, 1);
 	preferences->app = gossip_app_get ();
 
-	glade = gossip_glade_get_file (GLADEDIR "/main.glade",
-				     "preferences_dialog",
-				     NULL,
-				     "preferences_dialog", &preferences->dialog,
-				     "notebook", &preferences->notebook,
-				     "sound_checkbutton", &preferences->sound_checkbutton,
-				     "silent_busy_checkbutton", &preferences->silent_busy_checkbutton,
-				     "silent_away_checkbutton", &preferences->silent_away_checkbutton,
-				     "smileys_checkbutton", &preferences->smileys_checkbutton,
-				     "separate_windows_checkbutton", &preferences->separate_windows_checkbutton,
-				     "spell_checker_vbox", &preferences->spell_checker_vbox,
-				     "spell_checker_checkbutton", &preferences->spell_checker_checkbutton,
-				       "spell_checker_treeview", &preferences->spell_checker_treeview,
-				     NULL);
+	glade = gossip_glade_get_file (
+		GLADEDIR "/main.glade",
+		"preferences_dialog",
+		NULL,
+		"preferences_dialog", &preferences->dialog,
+		"notebook", &preferences->notebook,
+		"sound_checkbutton", &preferences->sound_checkbutton,
+		"silent_busy_checkbutton", &preferences->silent_busy_checkbutton,
+		"silent_away_checkbutton", &preferences->silent_away_checkbutton,
+		"smileys_checkbutton", &preferences->smileys_checkbutton,
+		"separate_windows_checkbutton", &preferences->separate_windows_checkbutton,
+		"spell_checker_vbox", &preferences->spell_checker_vbox,
+		"spell_checker_checkbutton", &preferences->spell_checker_checkbutton,
+		"spell_checker_treeview", &preferences->spell_checker_treeview,
+		"theme_combobox", &preferences->theme_combobox,
+		NULL);
 
 	g_object_add_weak_pointer (G_OBJECT (preferences->dialog),
 				   (gpointer) &preferences);
 	
+	preferences_themes_setup (preferences);
+
 	preferences_setup_widgets (preferences);
 	
 	preferences_languages_setup (preferences);
@@ -730,10 +883,12 @@ gossip_preferences_show (void)
 	if (gossip_spell_supported ()) {
 		GtkWidget *spell_checking_page;
 
-		spell_checking_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences->notebook), 1);
+		spell_checking_page = gtk_notebook_get_nth_page (
+			GTK_NOTEBOOK (preferences->notebook), 1);
 		gtk_widget_show (spell_checking_page);
 	}
 
-	gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog), GTK_WINDOW (gossip_app_get_window ()));
+	gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog),
+				      GTK_WINDOW (gossip_app_get_window ()));
 	gtk_widget_show (preferences->dialog);
 }
