@@ -40,12 +40,13 @@
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
+#define DEBUG_MSG(x)  
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
 #define COMPOSING_STOP_TIMEOUT 5
 
-#define d(x)
-
 
 struct _GossipPrivateChatPriv {
         GtkWidget        *widget;
@@ -89,7 +90,7 @@ static void           private_chat_connected_cb                 (GossipSession
 								 GossipPrivateChat      *chat);
 static void           private_chat_disconnected_cb              (GossipSession          *session,
 								 GossipPrivateChat      *chat);
-static void           private_chat_composing_event_cb           (GossipSession          *session,
+static void           private_chat_composing_cb                 (GossipSession          *session,
 								 GossipContact          *contact,
 								 gboolean                composing,
 								 GossipChat             *chat);
@@ -152,7 +153,7 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 	
         private_chat_create_gui (chat);
 
-	d(g_print ("PrivateChat: Connecting\n"));
+	DEBUG_MSG (("PrivateChat: Connecting"));
 
 	g_signal_connect_object (gossip_app_get_session (),
 				 "connected",
@@ -165,8 +166,8 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 				 chat, 0);
 	
 	g_signal_connect_object (gossip_app_get_session (),
-				 "composing-event",
-				 G_CALLBACK (private_chat_composing_event_cb),
+				 "composing",
+				 G_CALLBACK (private_chat_composing_cb),
 				 chat, 0);
 }
 
@@ -277,7 +278,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 	
 	if (priv->roster_resource &&
 	    g_ascii_strcasecmp (priv->roster_resource, roster_resource) == 0) {
-		d(g_print ("PrivateChat: Roster unchanged\n"));
+		DEBUG_MSG (("PrivateChat: Roster unchanged"));
 
 		if (!priv->locked_resource) {
 			priv->locked_resource = g_strdup (roster_resource);
@@ -286,7 +287,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 		return;
 	}
 	
-	d(g_print ("PrivateChat: New roster resource: %s\n", roster_resource));
+	DEBUG_MSG (("PrivateChat: New roster resource: %s", roster_resource));
 	
 	g_free (priv->roster_resource);
 	priv->roster_resource = g_strdup (roster_resource);
@@ -300,7 +301,7 @@ private_chat_send (GossipPrivateChat *chat,
                    const gchar       *msg)
 {
 	GossipPrivateChatPriv *priv;
-	GossipMessage         *m;
+	GossipMessage         *message;
 
 	priv = chat->priv;
 
@@ -319,24 +320,25 @@ private_chat_send (GossipPrivateChat *chat,
 
 	private_chat_update_locked_resource (chat);
 	
-	m = gossip_message_new (GOSSIP_MESSAGE_TYPE_NORMAL, priv->contact);
+	message = gossip_message_new (GOSSIP_MESSAGE_TYPE_NORMAL, priv->contact);
 
 	if (priv->locked_resource) {
-		gossip_message_set_explicit_resource (m, priv->locked_resource);
+		gossip_message_set_explicit_resource (message, priv->locked_resource);
 	}
 	
-	gossip_message_set_body (m, msg);
-	gossip_message_request_composing (m);
+	gossip_message_set_body (message, msg);
+	gossip_message_request_composing (message);
+	gossip_message_set_sender (message, priv->own_contact);
 
-	gossip_log_message (priv->own_contact, m, FALSE);
+	gossip_log_message_for_contact (message, FALSE);
 
 	gossip_chat_view_append_message_from_self (GOSSIP_CHAT (chat)->view, 
-						   m, 
+						   message, 
 						   priv->own_contact);
 
-	gossip_session_send_message (gossip_app_get_session (), m);
+	gossip_session_send_message (gossip_app_get_session (), message);
 	
-	g_object_unref (m);
+	g_object_unref (message);
 }
 
 static void
@@ -415,7 +417,7 @@ private_chat_contact_presence_updated (gpointer           not_used,
 		return;
 	}
 
-	d(g_print ("PrivateChat: Presence update for contact:'%s'\n", 
+	DEBUG_MSG (("PrivateChat: Presence update for contact:'%s'", 
 		   gossip_contact_get_id (contact)));
 
 	g_signal_emit_by_name (chat, "status-changed");
@@ -432,6 +434,7 @@ private_chat_contact_presence_updated (gpointer           not_used,
 						       msg);
 			g_free (msg);
 		}
+
 		priv->is_online = FALSE;
 
 		g_signal_emit_by_name (chat, "composing", FALSE);
@@ -440,12 +443,13 @@ private_chat_contact_presence_updated (gpointer           not_used,
 		if (!priv->is_online) {
 			gchar *msg;
 		
-			msg = g_strdup_printf (_("%s comes online"),
+			msg = g_strdup_printf (_("%s has come online"),
 					       gossip_contact_get_name (priv->contact));
 			gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
 						       msg);
 			g_free (msg);
 		}
+
 		priv->is_online = TRUE;
 	}
 }
@@ -526,10 +530,10 @@ private_chat_disconnected_cb (GossipSession     *session,
 }
 
 static void
-private_chat_composing_event_cb (GossipSession *session,
-				 GossipContact *contact,
-				 gboolean       composing,
-				 GossipChat    *chat)
+private_chat_composing_cb (GossipSession *session,
+			   GossipContact *contact,
+			   gboolean       composing,
+			   GossipChat    *chat)
 {
 	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
@@ -540,7 +544,7 @@ private_chat_composing_event_cb (GossipSession *session,
 	priv   = p_chat->priv;
 
 	if (gossip_contact_equal (contact, priv->contact)) {
-		d(g_print ("PrivateChat: Contact:'%s' %s typing\n",
+		DEBUG_MSG (("PrivateChat: Contact:'%s' %s typing",
 			   gossip_contact_get_name (contact),
 			   composing ? "is" : "is not"));
 
@@ -608,13 +612,15 @@ private_chat_input_key_press_event_cb (GtkWidget         *widget,
 		/* Newline for shift-enter. */
 		return FALSE;
 	}
-	else if (event->keyval == GDK_Page_Up) {
+	else if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
+		 event->keyval == GDK_Page_Up) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->text_view_sw));
 		gtk_adjustment_set_value (adj, adj->value - adj->page_size);
 		
 		return TRUE;
 	}
-	else if (event->keyval == GDK_Page_Down) {
+	else if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
+		 event->keyval == GDK_Page_Down) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->text_view_sw));
 		val = MIN (adj->value + adj->page_size, adj->upper - adj->page_size);  
 		gtk_adjustment_set_value (adj, val);
@@ -667,7 +673,7 @@ private_chat_get_name (GossipChat *chat)
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	return priv->name;
 }
@@ -678,22 +684,19 @@ private_chat_get_tooltip (GossipChat *chat)
 	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 	GossipContact         *contact;
-	gchar                 *str;
 	const gchar           *status;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	contact = gossip_chat_get_contact (chat);
 	status = gossip_contact_get_status (contact);
 
-	str = g_strdup_printf ("%s\n%s",
-			       gossip_contact_get_id (contact),
-			       status);
-
-	return str;
+	return g_strdup_printf ("%s\n%s",
+				gossip_contact_get_id (contact),
+				status);
 }
 
 GdkPixbuf *
@@ -706,7 +709,7 @@ private_chat_get_status_pixbuf (GossipChat *chat)
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	contact = gossip_chat_get_contact (chat);
 
@@ -722,7 +725,7 @@ private_chat_get_contact (GossipChat *chat)
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	return priv->contact;
 }
@@ -736,7 +739,7 @@ private_chat_get_own_contact (GossipChat *chat)
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	return priv->own_contact;
 }
@@ -759,7 +762,7 @@ private_chat_get_widget (GossipChat *chat)
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv   = p_chat->priv;
+	priv = p_chat->priv;
 
 	return priv->widget;
 }
@@ -836,28 +839,28 @@ gossip_private_chat_get_history (GossipPrivateChat *chat, gint lines)
 
 void
 gossip_private_chat_append_message (GossipPrivateChat *chat, 
-				    GossipMessage     *m)
+				    GossipMessage     *message)
 {
 	GossipPrivateChatPriv *priv;
 	GossipContact         *sender;
+	GossipChatroomInvite  *invite;
 	const gchar           *resource;
-	const gchar           *invite;
+	const gchar           *subject;
 	
         g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-        g_return_if_fail (GOSSIP_IS_MESSAGE (m));
+        g_return_if_fail (GOSSIP_IS_MESSAGE (message));
 
 	priv = chat->priv;
 	
-	d(g_print ("PrivateChat: Appending message ('%s')\n",
-		   gossip_contact_get_name (gossip_message_get_sender (m))));
+	DEBUG_MSG (("PrivateChat: Appending message ('%s')",
+		   gossip_contact_get_name (gossip_message_get_sender (message))));
 
-	sender = gossip_message_get_sender (m);
+	sender = gossip_message_get_sender (message);
 	if (!gossip_contact_equal (priv->contact, sender)) {
 		return;
 	}
 
-	resource = gossip_message_get_explicit_resource (m);
-
+	resource = gossip_message_get_explicit_resource (message);
 	if (resource) {
 		gboolean is_different;
 
@@ -869,15 +872,24 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 		}
 	}
 
-	gossip_log_message (priv->own_contact, m, TRUE);
+	gossip_log_message_for_contact (message, TRUE);
 
-	invite = gossip_message_get_invite (m);
+	subject = gossip_message_get_subject (message);
+ 	if (subject) {
+		gchar *str;
+
+		str = g_strdup_printf (_("Subject: %s"), subject);
+		gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, str);
+		g_free (str);
+	}
+
+	invite = gossip_message_get_invite (message);
 	if (invite) {
 		gossip_chat_view_append_invite (GOSSIP_CHAT (chat)->view,
-						m);
+						message);
 	} else {
 		gossip_chat_view_append_message_from_other (GOSSIP_CHAT (chat)->view,
-							    m,
+							    message,
 							    priv->own_contact);
 	}
 
@@ -885,5 +897,5 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 		gossip_sound_play (GOSSIP_SOUND_CHAT);
 	}
 
-	g_signal_emit_by_name (chat, "new-message");
+	g_signal_emit_by_name (chat, "new-message", message);
 }
